#ifndef _TRACKER_ABSTRACTAPVANALYSISFACTORY_H
#define _TRACKER_ABSTRACTAPVANALYSISFACTORY_H

#include "ApvAnalysis.h"


#include<vector>
#include<map>
#include<algorithm>
#include<string>


class DetUnit;



class ApvAnalysisFactory {
 public:
  typedef map < const DetUnit*, vector <ApvAnalysis *> > MapNumberingSchemaForAPVs;
  typedef map < const ApvAnalysis *,int > MapNumberingPosition;
  typedef map < const ApvAnalysis *, const DetUnit *> InverseMap;

  virtual void addDetUnitAndConstructApvs(const DetUnit *) = 0;

  //
  // Force a myName method, to be uswed for autoregistration
  //

  virtual string myName() = 0;

  vector <ApvAnalysis *> apvs(const DetUnit *);
  vector <ApvAnalysis *> allApvs();
  vector <const DetUnit *> allDetUnits();
  //
  // Access detUnit and position inside it (0..3 or 0..5, used to compute the bias)
  //
  const DetUnit* detUnit(const ApvAnalysis *);
  int positionInDetUnit(const ApvAnalysis *);
   
  void deleteApv(ApvAnalysis *);  
  void deleteApvOfDetUnit(const DetUnit *);  
  void clearAll();

  //
  // Override On demand mechanism...
  //
  void constructAllApvs();
  void constructAllApvs(vector <DetUnit *>);

  //
  //real action
  //

  virtual ApvAnalysis::StripDigisType doIt(const DetUnit*) = 0;

  //
  // As requested by Ian 10/04/2002
  //

  virtual void updateCalibration(){}
  virtual bool nextEvent(){return true;}

 protected:

  //
  // utility method to bias the strip number
  //
  virtual void addToOutPut(ApvAnalysis::StripDigisType&, ApvAnalysis::StripDigisType, int);

  virtual void updateMap(const DetUnit *);

  MapNumberingSchemaForAPVs theMap;
  MapNumberingPosition theMapPos;
  InverseMap theMapInverse;
};


#endif
