#ifndef VIEWER_Histogrammer
#define VIEWER_Histogrammer

#include "TH1.h"
#include "TProfile.h"
#include "TObjArray.h"
#include <map>
#include <vector>

#include "HistosForAPV.h"

class Histogrammer {

public:

   static Histogrammer* Instance();

   void  addAPVHistos(HistosForAPV* fhAPV);
   void  bookGenInfoHistograms();
   void  bookTHHistogram(int ival);
   void  bookRawHistograms();
   void  bookPedHistograms();
   void  bookCalibHistograms();

   void  fillRawHistograms(int iapv, const vector<unsigned short>& raw);
   void  fillPedHistograms(int iapv, int flg, ApvAnalysis::SubtractedSignalType& ped,
			               ApvAnalysis::SubtractedSignalType& noise, 
                                 	      TkApvMask::MaskType& mask);
   void  fillCMHistograms(int iapv, const vector<float>& cmode);
   void  fillCalibHistograms(int iapv, float time, 
	      ApvAnalysis::SubtractedSignalType& signal);
   void  saveAllHistograms(int itype);
   void  clearAllHistograms();

   TH1F* getRawHistogram(int iapv);
   TProfile* getPedHistogram(int iapv);
   TH1F* getRNoiseHistogram(int iapv);
   TH1F* getCMNoiseHistogram(int iapv);
   TH1F* getCMHistogram(int iapv);
   TH1F* getCalibHistogram(int iapv, int istr);
   TH1F* getTHHistogram(int ival);
   void  fillGeneralInfoHistogram(int year, int month, int date, int hour, 
                                  int min, int sec, int napv, int mode);
   void  fillTHHistogram(int ival, int ibin, float th_val);
//   void  printPointer() { cout << "Pointer = " << fTHHistoList << endl;}

protected:
    Histogrammer();
   ~Histogrammer();

private: 
   
   static Histogrammer* _instance;

   map<int, HistosForAPV*> fHistosMap;
   int                    fnAPVs;
   TH1F*  fGenInfoHisto;
   TObjArray* fTHHistoList;
};
#endif












