#include <iostream.h>
#include <strstream.h>
#include "Histogrammer.h" 
//
//Initialization
//
Histogrammer* Histogrammer::_instance = 0;
//
// Despatcher
//
Histogrammer* Histogrammer::Instance() {
  if (_instance == 0) {
      _instance = new Histogrammer();
   }
   return _instance;
}
//
// Constructor  
//
Histogrammer::Histogrammer() : fGenInfoHisto(0), fTHHistoList(0) {
  cout << "Constructiong Histogrammer ..." << endl;
  fHistosMap.clear();
  //  fTHHistoList->SetOwner(kTRUE);
  //  cout << " TList::GetSize() " << fTHHistoList->GetSize() << endl;
}
//
// -- Destructor
//
Histogrammer::~Histogrammer() {
  clearAllHistograms();
}
//
//  Load Histos objects
//
void Histogrammer::addAPVHistos(HistosForAPV* fhAPV) {
  fHistosMap.insert( pair<int, HistosForAPV*>(fhAPV->getType(), fhAPV) );
  cout << " Adding Histo object " << fHistosMap.size() << endl;
}
//
// -- Create General Info Histograms
//
void Histogrammer::bookGenInfoHistograms() {
  if (fGenInfoHisto) {
    fGenInfoHisto->Reset();
  } else {
    fGenInfoHisto = new TH1F ("General_Info", "General Informations",
                               10, 0.5, 10.5);
  }
}
//
// -- Create Temperature and Humidity Histograms
//
void  Histogrammer::bookTHHistogram(int ival) {
  // Create the TList if it is not already created 
  //  if (fTHHistoList == 0) fTHHistoList = new TList();
  if (fTHHistoList == 0) fTHHistoList = new TObjArray(12,0);
  int  itag;
  char hName[40];
  char hTitle[45];
  ostrstream   str_hname(hName,40);
  ostrstream   str_htitle(hTitle,45);
  cout << "booking histogram, ival = " << ival << endl;
  if (ival > 20) {
    // Humidity 
    itag  = ival - 20 ;
    str_hname << "Sens_" << itag << "_Humidity_vs_Time" << ends;
    str_htitle << "Humidity vs Time " << "for Sensor #" << itag << ends;
  } else {
    itag  = ival;
    str_hname << "Sens_" << itag << "_Temperature_vs_Time" << ends;
    str_htitle << "Temperature vs Time " << "for Sensor #" << itag << ends;
  }
  TH1F* fTHHisto = new TH1F (hName, hTitle, 600, 0.0, 1800.0); 
  // Define the colour of the plot
  if (itag < 8) { 
    fTHHisto->SetLineColor(itag+1);
  } else {
    fTHHisto->SetLineColor(itag+30);
  }
  //  fTHHisto->GetXaxis()->SetTimeDisplay(1);
  fTHHisto->GetXaxis()->SetTimeFormat("%H:%M");
  if (fTHHistoList) {
     fTHHistoList->AddAt(fTHHisto, itag);  
  } else {
    cout << "Strange!" << endl;
  }
}
//
// -- Create Raw Histograms
//
void Histogrammer::bookRawHistograms() {
  this->bookGenInfoHistograms();
  for (map<int, HistosForAPV*>::const_iterator it =  fHistosMap.begin() ; 
       it != fHistosMap.end(); it++) {
    int iapv = (*it).first;
    cout<<"Booking histogram files for APV ID  "<< iapv <<endl;
    HistosForAPV* fHistos = (*it).second;
    fHistos->bookHistograms(1); 
  }
}
//
// -- Create Pedestal Histograms
//
void Histogrammer::bookPedHistograms() {
  this->bookGenInfoHistograms();
  for (map<int, HistosForAPV*>::const_iterator it =  fHistosMap.begin() ; 
       it != fHistosMap.end(); it++) {
    int iapv = (*it).first;
    cout<<" Booking histograms for APV ID "<< iapv <<endl;
    HistosForAPV* fHistos = (*it).second;
    fHistos->bookHistograms(2); 
  }
}
//
// -- Create Pedestal Histograms
//
void Histogrammer::bookCalibHistograms() {
  this->bookGenInfoHistograms();
  for (map<int, HistosForAPV*>::const_iterator it =  fHistosMap.begin() ; 
       it != fHistosMap.end(); it++) {
    int iapv = (*it).first;
    cout<<" Booking histograms for APV ID "<< iapv <<endl;
    HistosForAPV* fHistos = (*it).second;
    fHistos->bookHistograms(3); 
  }
}
//
// -- Fill General Information Histogram
//
void Histogrammer::fillGeneralInfoHistogram(int year, int month, int date, 
			  int hour, int min, int sec, int napv, int mode){
  if (fGenInfoHisto) {
    fGenInfoHisto->SetBinContent(1, static_cast<float>(year));
    fGenInfoHisto->SetBinContent(2, static_cast<float>(month));
    fGenInfoHisto->SetBinContent(3, static_cast<float>(date));
    fGenInfoHisto->SetBinContent(4, static_cast<float>(hour));
    fGenInfoHisto->SetBinContent(5, static_cast<float>(min));
    fGenInfoHisto->SetBinContent(6, static_cast<float>(sec));
    fGenInfoHisto->SetBinContent(7, static_cast<float>(napv));
    fGenInfoHisto->SetBinContent(8, static_cast<float>(mode));
  }
}
//
// -- Fill Temperature and Humidity Hostogram
//
void Histogrammer::fillTHHistogram(int ival, int ibin, float th_val){
  int itag;
  if (ival > 20) {
    itag = ival - 20;
  } else {
    itag = ival;
  }
  TH1F* fTHHisto = dynamic_cast<TH1F*>(fTHHistoList->At(itag));
  if (fTHHisto)  fTHHisto->Fill(ibin,th_val); 
}
//
// -- Fill  Raw Histograms
//
void Histogrammer::fillRawHistograms(int iapv, const vector<unsigned short>& raw) {
  HistosForAPV* fHistos = fHistosMap[iapv];
  if (fHistos) {
    fHistos->fillRawHistos(raw);
  }
}
//
// -- Fill  Pedestal  Histograms
//
void Histogrammer::fillPedHistograms(int iapv, int flg, ApvAnalysis::SubtractedSignalType& ped,
			               ApvAnalysis::SubtractedSignalType& noise, 
                                 	      TkApvMask::MaskType& mask){
                                          
  HistosForAPV* fHistos = fHistosMap[iapv];
  if (fHistos) {
    fHistos->fillHistos(flg, ped, noise, mask);
  }
}
//
// -- Fill  Common Mode Histogram
//
void Histogrammer::fillCMHistograms(int iapv, const vector<float>& cmode){
  HistosForAPV* fHistos = fHistosMap[iapv];
  if (fHistos) {
    fHistos->fillHistos(cmode);
  }
}
//
// -- Fill  Calibration Histogram
//
void Histogrammer::fillCalibHistograms(int iapv, float time, 
	       ApvAnalysis::SubtractedSignalType& signal){
  HistosForAPV* fHistos = fHistosMap[iapv];
  if (fHistos) {
    fHistos->fillHistos(time, signal);
  }
}
//
// -- Get Raw Histogram for a specific channel
//
TH1F* Histogrammer::getRawHistogram(int iapv) {
  TH1F* hraw = 0;
  int nSize = fHistosMap.size();
  if (nSize != 0 && iapv < nSize) {
    hraw = fHistosMap[iapv]->getRawHisto();
  }
  return hraw;
}
//
// -- Get Pedestal Histogram for a specific channel
//
TProfile* Histogrammer::getPedHistogram(int iapv) {
  TProfile* hped = 0;
  int nSize = fHistosMap.size();
  if (nSize != 0 && iapv < nSize) {
    hped = fHistosMap[iapv]->getPed1Histo();
  }
  return hped;
}
//
// -- Get Raw Noise Histogram for a specific channel
//
TH1F* Histogrammer::getRNoiseHistogram(int iapv) {
  TH1F* hrnoise= 0;
  int nSize = fHistosMap.size();
  if (nSize != 0 && iapv < nSize) {
    hrnoise = fHistosMap[iapv]->getRNoise1Histo();
  }
  return hrnoise;
}
//
// -- Get CM Noise Histogram for a specific channel
//
TH1F* Histogrammer::getCMNoiseHistogram(int iapv) {
  TH1F*  hcmnoise = 0;
  int nSize = fHistosMap.size();
  if (nSize != 0 && iapv < nSize) {
    hcmnoise = fHistosMap[iapv]->getCMNoise1Histo();
  }
  return hcmnoise;
}
//
// -- Get Common Mode Histogram for a specific channel
//
TH1F* Histogrammer::getCMHistogram(int iapv) {
  TH1F*  hcm = 0;
  int nSize = fHistosMap.size();
  if (nSize != 0 && iapv < nSize) {
    hcm = fHistosMap[iapv]->getCMHisto();
  }
  return hcm;
}
//
// -- Get Calibration Histogram for a specific channel and specific Strip
//
TH1F* Histogrammer::getCalibHistogram(int iapv, int istr) {
  TH1F*  hcal = 0;
  int nSize = fHistosMap.size();
  if (nSize != 0 && iapv < nSize) {
    hcal = fHistosMap[iapv]->getCalibHisto(istr);
  }
  return hcal;
}
//
// -- Get TH Histogram
//
TH1F* Histogrammer::getTHHistogram(int ival){
  int itag;
  if (fTHHistoList->GetSize() != 0) {
    if (ival > 20) {
    // Humidity 
      itag  = ival - 20 ; 
    } else {
      itag = ival;
    }
    TH1F* fTHHisto = dynamic_cast<TH1F*>(fTHHistoList->At(itag));
    if (fTHHisto) {
      return fTHHisto;
    }
  }
  return 0;
}
//
// Save Histograms in a Root File
//
void Histogrammer::saveAllHistograms(int itype) {
  if (fGenInfoHisto) fGenInfoHisto->Write();
  for (map<int, HistosForAPV*>::const_iterator it =  fHistosMap.begin() ; 
       it != fHistosMap.end(); it++) {
    HistosForAPV* fHistos = (*it).second;
    if (fHistos) {
      cout << " Saving Histograms for APV ID "<< (*it).first << endl;
      fHistos->saveHistos(itype);
    }
  }
  // Delete Temperature and Humidity Histograms
  if (fTHHistoList != 0) {
    for (int i = 0; i < fTHHistoList->GetSize(); i++) {
      TH1F* fTHHisto = dynamic_cast<TH1F*>(fTHHistoList->At(i+1));
      if (fTHHisto) fTHHisto->Write();
    }
  }
}
//
// Clear All Histograms 
//
void Histogrammer::clearAllHistograms() {
  // Delete General Purpose Histograms
  if (fGenInfoHisto) {
    delete fGenInfoHisto;
    fGenInfoHisto = 0;
  }
  // Delete APV related Histograms
  for (map<int, HistosForAPV*>::const_iterator it =  fHistosMap.begin() ; 
       it != fHistosMap.end(); it++) {
    HistosForAPV* fHistos = (*it).second;
    if (fHistos) {
      cout << " Deleting Histograms for APV ID " << (*it).first << endl;
      fHistos->clearHistograms();
    }
  }
  fHistosMap.clear();
  // Delete Temperature and Humidity Histograms
  if (fTHHistoList != 0) {
    for (int i = 0; i < fTHHistoList->GetSize(); i++) {
      TH1F* fTHHisto = dynamic_cast<TH1F*>(fTHHistoList->At(i+1));
      if (fTHHisto) delete fTHHisto;
    }
    delete fTHHistoList;
    fTHHistoList = 0;
  }
}
