#include <iostream.h>
#include "HistosForAPV.h" 
//
// Constructor  
//
HistosForAPV::HistosForAPV(int id_apv):fIdAPV(id_apv) {
  cout << " Creating  HistosForAPV for APV Number " << fIdAPV << endl;
  initHistograms();
}
//
// -- Destructor
//
HistosForAPV::~HistosForAPV() {
  if (fPedHistoBooked || fCalibHistoBooked) clearHistograms();
}
//
// -- Delete Histograms Destructor
//
void HistosForAPV::clearHistograms() {
  if (fRawHisto)        delete    fRawHisto;
  if (fInitPed1Histo)   delete    fInitPed1Histo;
  if (fInitPed2Histo)   delete    fInitPed2Histo;
  if (fInitNoise1Histo) delete    fInitNoise1Histo;
  if (fInitNoise2Histo) delete    fInitNoise2Histo;
  if (fPed1Histo)       delete    fPed1Histo;
  if (fPed2Histo)       delete    fPed2Histo;
  if (fRNoise1Histo)    delete    fRNoise1Histo;
  if (fRNoise2Histo)    delete    fRNoise2Histo;
  if (fRNoise3Histo)    delete    fRNoise3Histo;
  if (fSignal1Histo)    delete    fSignal1Histo; 
  if (fSignal2Histo)    delete    fSignal2Histo; 
  if (fSignal3Histo)    delete    fSignal3Histo; 
  if (fCMNoise1Histo)   delete    fCMNoise1Histo;
  if (fCMNoise2Histo)   delete    fCMNoise2Histo;
  if (fCMNoise3Histo)   delete    fCMNoise3Histo;
  if (fMask1Histo)      delete    fMask1Histo;
  if (fMask2Histo)      delete    fMask2Histo;
  if (fCMHisto)         delete    fCMHisto; 

  for (unsigned int i = 0; i < fCalibHistoVec.size() ; i++) {
    TH1F* hraw = fCalibHistoVec[i];
    delete hraw;
  }
  fCalibHistoVec.clear();
  initHistograms();
}
//
// -- Init Histograms
//
void HistosForAPV::initHistograms() {
  fRawHisto      = 0;
  fInitPed1Histo = 0;
  fInitPed2Histo = 0;
  fInitNoise1Histo = 0;
  fInitNoise2Histo = 0;
  fPed1Histo     = 0;
  fPed2Histo     = 0;
  fRNoise1Histo  = 0;
  fRNoise2Histo  = 0;
  fRNoise3Histo  = 0;
  fSignal1Histo  = 0;
  fSignal2Histo  = 0;
  fSignal3Histo  = 0;
  fCMNoise1Histo = 0;
  fCMNoise2Histo = 0;
  fCMNoise3Histo = 0;
  fMask1Histo    = 0;
  fMask2Histo    = 0;
  fCMHisto       = 0;
  fPedHistoBooked = false;

  fCalibHistoVec.clear();
  fCalibHistoBooked = false;

}
//
// -- Create Histograms
//
void HistosForAPV::bookHistograms(int itype) {
  char name[80],title[80];
  // Book Raw Histograms 
  if (fRawHisto == 0) {
    sprintf(name,"APV_%.1d_rawdata",fIdAPV+1);
    sprintf(title,"APV #%.1d Raw Data ",fIdAPV+1);
    fRawHisto = new TH1F(name,title,140,0.5,140.5);
    fRawHisto->SetStats(0);
  } else {
    fRawHisto->Reset();
  }

  if (itype > 1) {
    if (fPedHistoBooked) {
      fInitPed1Histo->Reset();
      fInitPed2Histo->Reset();
      fInitNoise1Histo->Reset();
      fInitNoise2Histo->Reset();
      fPed1Histo->Reset();
      fPed2Histo->Reset();
      fRNoise1Histo->Reset();
      fRNoise2Histo->Reset();
      fRNoise3Histo->Reset();
      fSignal1Histo->Reset();
      fSignal2Histo->Reset();
      fSignal3Histo->Reset();
      fCMNoise1Histo->Reset();
      fCMNoise2Histo->Reset();
      fCMNoise3Histo->Reset();
      fMask1Histo->Reset();
      fMask2Histo->Reset();
      fCMHisto->Reset();
    } else {

      sprintf(name,"APV_%.1d_Iped_vs_str",fIdAPV+1);
      sprintf(title,"APV #%.1d Initial Pedestal vs Strip ",fIdAPV+1);
      fInitPed1Histo = new TH1F(name,title,128,0.5,128.5);

      sprintf(name,"APV_%.1d_Ipedestal",fIdAPV+1);
      sprintf(title,"APV #%.1d Initial Pedestal ",fIdAPV+1);
      fInitPed2Histo = new TH1F(name,title,100,0.0,500.0);

      sprintf(name,"APV_%.1d_Inoise_vs_str",fIdAPV+1);
      sprintf(title,"APV #%.1d Initial Noise vs Strip ",fIdAPV+1);
      fInitNoise1Histo = new TH1F(name,title,128,0.5,128.5);

      sprintf(name,"APV_%.1d_Inoise",fIdAPV+1);
      sprintf(title,"APV #%.1d Initial Noise ",fIdAPV+1);
      fInitNoise2Histo = new TH1F(name,title,100,0.0,20.0);

      sprintf(name,"APV_%.1d_ped_vs_str",fIdAPV+1);
      sprintf(title,"APV #%.1d Pedestal vs Strip ",fIdAPV+1);
      fPed1Histo = new TProfile(name,title,128,0.5,128.5,0.0,500.0);
      fPed1Histo->SetStats(0);

      sprintf(name,"APV_%.1d_pedestal",fIdAPV+1);
      sprintf(title,"APV #%.1d Pedestal ",fIdAPV+1);
      fPed2Histo = new TH1F(name,title,100,0.0,500.0);

      sprintf(name,"APV_%.1d_rnoise_vs_str",fIdAPV+1);
      sprintf(title,"APV #%.1d Raw Noise vs Strip ",fIdAPV+1);
      fRNoise1Histo = new TH1F(name,title,128,0.5,128.5);
      fRNoise1Histo->SetStats(0);

      sprintf(name,"APV_%.1d_rnoise_vs_str_prof",fIdAPV+1);
      sprintf(title,"APV #%.1d Raw Noise vs Strip ",fIdAPV+1);
      fRNoise3Histo = new TProfile(name,title,128,0.5,128.5,0.0,25.0);

      sprintf(name,"APV_%.1d_rnoise",fIdAPV+1);
      sprintf(title,"APV #%.1d Raw Noise ",fIdAPV+1);
      fRNoise2Histo = new TH1F(name,title,100,0.0,10.0);

      sprintf(name,"APV_%.1d_signal_vs_str",fIdAPV+1);
      sprintf(title,"APV #%.1d Signal vs Strip ",fIdAPV+1);
      fSignal1Histo = new TH1F(name,title,128,0.5,128.5);

      sprintf(name,"APV_%.1d_signal_vs_str_prof",fIdAPV+1);
      sprintf(title,"APV #%.1d Signal vs Strip ",fIdAPV+1);
      fSignal3Histo = new TProfile(name,title,128,0.5,128.5,-10.0,10.0);

      sprintf(name,"APV_%.1d_signal",fIdAPV+1);
      sprintf(title,"APV #%.1d Signal ",fIdAPV+1);
      fSignal2Histo = new TH1F(name,title,200,-10.0,10.0);

      sprintf(name,"APV_%.1d_cmnoise_vs_str",fIdAPV+1);
      sprintf(title,"APV #%.1d CM Noise vs Strip ",fIdAPV+1);
      fCMNoise1Histo = new TH1F(name,title,128,0.5,128.5);
      fCMNoise1Histo->SetStats(0);

      sprintf(name,"APV_%.1d_cmnoise_vs_str_prof",fIdAPV+1);
      sprintf(title,"APV #%.1d CM Noise vs Strip ",fIdAPV+1);
      fCMNoise3Histo = new TProfile(name,title,128,0.5,128.5,0.0,25.0);

      sprintf(name,"APV_%.1d_cmnoise",fIdAPV+1);
      sprintf(title,"APV #%.1d CM Noise ",fIdAPV+1);
      fCMNoise2Histo = new TH1F(name,title,100,0.0,10.0);

      sprintf(name,"APV_%.1d_mask_vs_str",fIdAPV+1);
      sprintf(title,"APV #%.1d Mask vs Strip ",fIdAPV+1);
      fMask1Histo = new TH2F(name,title,128,0.5,128.5,8,0.5,8.5);

      sprintf(name,"APV_%.1d_mask",fIdAPV+1);
      sprintf(title,"APV #%.1d Mask ",fIdAPV+1);
      fMask2Histo = new TH1F(name,title,9, -0.5, 8.5);

      sprintf(name,"APV_%.1d_cmode",fIdAPV+1);
      sprintf(title,"APV #%.1d Common Mode",fIdAPV+1);
      fCMHisto = new TH1F(name,title,100,-20.0, 20.0);

      fPedHistoBooked = true;
    }
  }
  if (itype == 3) {
    if (fCalibHistoBooked) {
      for (unsigned int i = 0; i < fCalibHistoVec.size() ; i++) {
        fCalibHistoVec[i]->Reset();
      }
    } else {
      for (unsigned int i = 0; i < 128; i++) {
        sprintf(name,"APV_%.1d_Strip_%.2d",fIdAPV+1,i+1);
        sprintf(title,"APV #%.1d Strip #%.2d Pulse Shape",fIdAPV+1,i+1);
        TH1F* hcalib = new TH1F(name,title,104,0.,325.);
        fCalibHistoVec.push_back(hcalib);
      }
      fCalibHistoBooked = true;
    }
  }
}
//
// -- Get Raw Histogram 
//
TH1F* HistosForAPV::getRawHisto() {
  return fRawHisto;
}
//
// -- Get Pedestal vs Strip Histogram
//
TProfile* HistosForAPV::getPed1Histo() {
  return fPed1Histo;
}
//
// -- Get Raw Noise vs Strip Histogram
//
TH1F* HistosForAPV::getRNoise1Histo() {
  return fRNoise1Histo;
}
//
// -- Get CM Noise vs Strip Histogram
//
TH1F* HistosForAPV::getCMNoise1Histo() {
  return fCMNoise1Histo;
}
//
// -- Get Common Mode Histogram
//
TH1F* HistosForAPV::getCMHisto() {
  return fCMHisto;
}
//
// -- Get Calibration Histogram for strip number 'istr'
//
TH1F* HistosForAPV::getCalibHisto(int istr) {
  TH1F* hCalib = fCalibHistoVec[istr];
  return hCalib;
}
//
// -- Get Type
//
int  HistosForAPV::getType() {
  return fIdAPV;
}
//
// -- Fill Raw Histogram
//
void HistosForAPV::fillRawHistos(const vector<unsigned short>& RData) {
  fRawHisto->Reset();
  for (unsigned int i = 0; i < RData.size(); i++) {
    fRawHisto->Fill((i+1), static_cast<float>(RData[i]));
  }
}
//
// -- Fill Common Mode Histogram
//
void HistosForAPV::fillHistos(const vector<float>& CMode) {
  for (unsigned int i = 0; i < CMode.size(); i++) {
    fCMHisto->Fill(CMode[i]);
  }
}
//
// -- Fill Pedestal Histograms 
//
void HistosForAPV::fillHistos(int flg, ApvAnalysis::SubtractedSignalType& ped,
			               ApvAnalysis::SubtractedSignalType& noise, 
                                 	      TkApvMask::MaskType& mask){

  if (flg == 1) {
    for (unsigned int i = 0; i < ped.size(); i++) {
      fInitPed1Histo->Fill((i+1), ped[i]);
      fInitPed2Histo->Fill(ped[i]);
      fInitNoise1Histo->Fill((i+1), noise[i]);
      fInitNoise2Histo->Fill(noise[i]);
      switch (mask[i]) {
      case TkApvMask::dead :
        {
         fMask1Histo->Fill(static_cast<float>(i+1), 4.0);
         fMask2Histo->Fill(4.0); 
         break;
        }        
      case TkApvMask::noisy :
        {
         fMask1Histo->Fill(static_cast<float>(i+1), 5.0);
         fMask2Histo->Fill(5.0); 
         break;
        }        
      case TkApvMask::ok :
        {
         fMask1Histo->Fill(static_cast<float>(i+1), mask[i]);
         fMask2Histo->Fill(mask[i]);
        }
      }
    }
  } else if (flg == 2) {
    fRNoise1Histo->Reset();
    for (unsigned int i = 0; i < ped.size(); i++) {
      fPed1Histo->Fill(static_cast<float>(i+1), ped[i]);
      fPed2Histo->Fill(ped[i]);
      if (fabs(noise[i]) < 100.0) {
        fRNoise1Histo->Fill((i+1), noise[i]);
        fRNoise3Histo->Fill(static_cast<float>(i+1), noise[i]);
      }
      fRNoise2Histo->Fill(noise[i]);
      switch (mask[i]) {
      case TkApvMask::dead :
        {
         fMask1Histo->Fill(static_cast<float>(i+1), 6.0);
         fMask2Histo->Fill(4.0); 
         break;
        }        
      case TkApvMask::noisy :
        {
         fMask1Histo->Fill(static_cast<float>(i+1), 7.0);
         fMask2Histo->Fill(5.0); 
         break;
        } 
      case TkApvMask::ok :     
        {  
         fMask1Histo->Fill(static_cast<float>(i+1), mask[i]);
         fMask2Histo->Fill(mask[i]);
        }
      }
    }
  } else if (flg == 3) {
    fCMNoise1Histo->Reset();
    //      fSignal1Histo->Reset();    
    for (unsigned int i = 0; i < noise.size(); i++) {
      //      fSignal1Histo->Fill((i+1), ped[i]);
      //      fSignal2Histo->Fill(ped[i]);
      if (fabs(noise[i]) < 100.0) {
        fCMNoise1Histo->Fill((i+1), noise[i]);
        fCMNoise3Histo->Fill(static_cast<float>(i+1), noise[i]);
      }
      fCMNoise2Histo->Fill(noise[i]);
      fMask1Histo->Fill(static_cast<float>(i+1), mask[i]);
      fMask2Histo->Fill(mask[i]);
    }
  }
}
//
// -- Fill Calibration Histograms
//
void HistosForAPV::fillHistos(float time, 
			      ApvAnalysis::SubtractedSignalType& signal) {
  
  for (unsigned int i = 0; i < signal.size() ; i++) {
    fCalibHistoVec[i]->Fill(time, signal[i]);
  }    
}
//
// -- Save Histograms
//
void HistosForAPV::saveHistos(int itype) {
  fRawHisto->Write();
  if (itype > 1) {
    fInitPed1Histo->Write();
    fInitPed2Histo->Write();
    fInitNoise1Histo->Write();
    fInitNoise2Histo->Write();
    fPed1Histo->Write();
    fPed2Histo->Write();
    fRNoise1Histo->Write();
    fRNoise2Histo->Write();
    fRNoise3Histo->Write();
    //    fSignal1Histo->Write();
    //    fSignal2Histo->Write();
    fCMNoise1Histo->Write();
    fCMNoise2Histo->Write();
    fCMNoise3Histo->Write();
    fMask1Histo->Write();
    fMask2Histo->Write();
    fCMHisto->Write();
  }
  if (itype == 3) {
    for (unsigned int i = 0; i < fCalibHistoVec.size() ; i++) {
      fCalibHistoVec[i]->Write();
    }    
  }
}

