#include "MTApvFactory.h"

#include "ReferenceApvMask.h"
#include "ReferenceApvAnalysis.h"
#include "ReferenceCommonModeCalculatorWithSignalRejection.h"
#include "ReferenceZeroSuppresser.h"
#include "ReferenceClusterZeroSuppresser.h"
#include "TkCommonMode.h"
#include "TkCommonModeTopology.h"
#include "ReferencePedestalCalculatorWithSignalRejection.h"
#include "ReferencePedestalCalculator.h"
#include "ReferenceNoiseCalculatorWithSignalRejection.h"

MTApvFactory* MTApvFactory::myInstance(0);

MTApvFactory* MTApvFactory::instance(){
  if (!myInstance) 
    myInstance = new MTApvFactory;
  return myInstance;
}
void MTApvFactory::setEventReader(ApvEventReader * in ){
  theEventReader = in;
}
ApvAnalysis* MTApvFactory::getApv(int in){
  if (theMap[in] == 0){
    ApvAnalysis* temp = constructApv();
    theMapReversed[temp]=in;
    theMap[in]=temp;
    cout <<" Constructed APV "<<temp<<endl;;
  }
  return theMap[in];
}
ApvAnalysis* MTApvFactory::constructApv(){
  ApvAnalysis* theAPV = new ReferenceApvAnalysis;
  
  ReferenceNoiseCalculatorWithSignalRejection* theNoise = new ReferenceNoiseCalculatorWithSignalRejection(20);
  ReferencePedestalCalculatorWithSignalRejection* thePedestal = new ReferencePedestalCalculatorWithSignalRejection(20);
  TkZeroSuppresser* theZS = new ReferenceClusterZeroSuppresser;
  ReferenceCommonModeCalculatorWithSignalRejection* theCM = new ReferenceCommonModeCalculatorWithSignalRejection;
  TkCommonModeTopology* theTopology = new TkCommonModeTopology(128,128);
  TkCommonMode* theCommonMode = new TkCommonMode;
  ReferenceApvMask* theMask = new ReferenceApvMask(theNoise);
  theCommonMode->setTopology(theTopology);
  theCM->setCM(theCommonMode);
  //
  // set inside the APV
  //
  theAPV->setZeroSuppresser(*theZS);
  theAPV->setCommonModeCalculator(*theCM);
  theAPV->setPedestalCalculator(*thePedestal);
  theAPV->setNoiseCalculator(*theNoise);
  theAPV->setMask(*theMask);
  if (theEventReader==0){
    cerr<<" ERROR: No event Reader Defined!"<<endl;
  }
  theAPV->setReader(*theEventReader);
  
  ApvAnalysis::SubtractedSignalType peds;
  TkApvMask::MaskType mask;
  ApvAnalysis::SubtractedSignalType noise;
  //  for (int i=0; i<128; i++){
  //    peds.push_back(0.);
  // mask.push_back(TkApvMask::ok);
  // noise.push_back(3.);
  //}
  //theNoise->setStripNoise(noise);
  //theMask->setMask(mask);
  //thePedestal->setPedestals(peds);
  //  thePedestal->status()->setReady();
  //  theNoise->status()->setReady();  
  return theAPV;
}

void MTApvFactory::clearAllCalibrations(){
  //
  // delete of all the APVs... I know it is brutal, but it works :)
  //
  theMap.clear();
  for (map<ApvAnalysis*,int>::iterator it = theMapReversed.begin(); it!=theMapReversed.end(); it++){
    //
    // delete the Apv ands all the classes I created
    //
        deleteApv((*it).first);
  }
  theMapReversed.clear();
}

void MTApvFactory::deleteApv(ApvAnalysis* apv){
  cout <<"Deleting APV "<<apv<<endl;
  delete &(apv->pedestalCalculator());
  delete &(apv->noiseCalculator());
  delete &(apv->mask());
  delete &(apv->commonModeCalculator().commonMode()->topology());
  delete (apv->commonModeCalculator().commonMode());
  delete &(apv->commonModeCalculator());
  delete apv;
}

