#include "MedianTT6CommonModeCalculator.h"
#include <vector>
#include <algorithm>

// SimpleConfigurable<float> MedianTT6CommonModeCalculator::cutToAvoidSignal(3., "CmnCalculator:cutToAvoidSignal");

// 
//  Constructors:
//
MedianTT6CommonModeCalculator::MedianTT6CommonModeCalculator(TkNoiseCalculator* noise_calc, TkApvMask* mask_calc) : 
  theNoiseCalculator(noise_calc),
  theApvMask(mask_calc),
  alreadyUsedEvent(false)
{
  if (0) cout << "Constructing MedianTT6CommonMode Calculator ..." << endl;
}

/*
MedianTT6CommonModeCalculator::MedianTT6CommonModeCalculator(TkNoiseCalculator* noise_calc, TkApvMask* mask_calc, float sig_cut) : 
  theNoiseCalculator(noise_calc),
  theApvMask(mask_calc),
  alreadyUsedEvent(false)
{
  if (0) cout << "Constructing TT6CommonMode Calculator ..." << endl;
  cutToAvoidSignal = sig_cut;
}
*/

//
//  Destructor 
//
MedianTT6CommonModeCalculator::~MedianTT6CommonModeCalculator() {
  if (0) cout << "Destructing MedianTT6CommonModeCalculator " << endl;
}
//
// Action :
//
ApvAnalysis::SubtractedSignalType MedianTT6CommonModeCalculator::doIt
                (ApvAnalysis::SubtractedSignalType indat) 
{
  ApvAnalysis::SubtractedSignalType out;
  calculateCommonMode(indat);
  
  int setNumber;
  for (unsigned int i=0; i<indat.size(); i++){
    setNumber = theTkCommonMode->topology().setOfStrip(i);
    out.push_back(indat[i] - theCommonModeValues[setNumber]);
  }  
  return out;
}
//
//  Calculation of Common Mode Values :
//
void MedianTT6CommonModeCalculator::calculateCommonMode
        (ApvAnalysis::SubtractedSignalType& indat) 
{ 
  if (alreadyUsedEvent == false) {
    alreadyUsedEvent = true;

    TkApvMask::MaskType strip_mask = theApvMask->mask();
    TkNoiseCalculator::NoiseType strip_noise = theNoiseCalculator->stripNoise();

    int nSet = theTkCommonMode->topology().numberOfSets();
    theCommonModeValues.clear();

    for (int i=0; i<nSet; i++){
      int initial   = theTkCommonMode->topology().initialStrips()[i];
      int final     = theTkCommonMode->topology().finalStrips()[i];
      ApvAnalysis::SubtractedSignalType tmp;
      for (int j = initial; j <= final; j++) {
	if (strip_mask[j] == TkApvMask::ok) tmp.push_back(indat[j]);
      }
      int nval = tmp.size()/2-1;
      float theCMval;
      if (nval>=0) {
	sort(tmp.begin(),tmp.end());
	theCMval = tmp[nval];
      } else {
	//        cout << "warning MedianTT6CommonModeCalculator: size is: " << tmp.size() << endl;
	theCMval = 0.;
      }
	theCommonModeValues.push_back(theCMval);
    }

    MedianTT6CommonModeCalculator::setCM(theCommonModeValues);
  }
}
//
// Define New Event
// 
void MedianTT6CommonModeCalculator::newEvent() {
  alreadyUsedEvent = false;
}













