#include "TT6ApvAnalysis.h"
#include "TT6PedestalCalculator.h"
#include "TT6CommonModeCalculator.h"
#include "ApvEventReader.h"
#include "TkNoiseCalculator.h"
#include "TkPedestalCalculator.h"
#include "TkApvMask.h"
#include "TkZeroSuppresser.h"
#include "TkCommonModeCalculator.h"
#include <algorithm>
#include <functional>
//
// Constructor
//
TT6ApvAnalysis::TT6ApvAnalysis(){
  cout << " Constructing TT6ApvAnalysis " << endl;
  nEvents = 0;
  theRawsignalFlag = false;
  thePedSubSignalFlag = false;
  theRawSignal.reserve(128);
  thePedSubSignal.reserve(128);
}

//
// Destructor
//
TT6ApvAnalysis::~TT6ApvAnalysis(){
  cout << " Destructing TT6ApvAnalysis " << endl;
}
//
// Set Parameters
//
void TT6ApvAnalysis::setParameters(int _nEventForInitialization, int _nEventForUpdate) {
  nEventForInitialization = _nEventForInitialization;
  nEventForUpdate = _nEventForUpdate;
}
//
// Access Raw Data
//
ApvAnalysis::RawSignalType  TT6ApvAnalysis::rawSignal(){
  // make some pre-processing ... !!! pay attention to ricursion !!
  if (!initialized) {
    updateCalibration();
  }
  if (!theRawsignalFlag) {
    if (theApvEventReader) {
      theRawSignal = theApvEventReader->rawSignal(*this);
      theRawsignalFlag = true;
    } else {
      cout << " Error ! No EventReader is defined!! " << endl;
    }
  }
  return theRawSignal;
}

//
//  get Pedestal subtracted data from the PedestalCalculator
//
ApvAnalysis::SubtractedSignalType  TT6ApvAnalysis::pedestalSubtractedSignal(){
  if (!thePedSubSignalFlag) {
    if (theTkPedestalCalculator) {
      thePedSubSignal = theTkPedestalCalculator->doIt(rawSignal());
      thePedSubSignalFlag = true;
    } else {
      cout << " Error ! No PedestalCalculator is defined!! " << endl;
    }  
  }
  return thePedSubSignal;
}

//
//  get Pedestal and CM subtracted data from the CommonModeCalculator
//
ApvAnalysis::SubtractedSignalType  TT6ApvAnalysis::CMPSubtractedSignal(){
  //  ApvAnalysis::RawSignalType        tempraw;
  //  ApvAnalysis::SubtractedSignalType temppsub, temp;
  ApvAnalysis::SubtractedSignalType temp;
  if ( theTkCommonModeCalculator && theTkPedestalCalculator) {
    //    tempraw = rawSignal();
    //    temppsub = theTkPedestalCalculator->doIt(tempraw);
    //    temppsub = pedestalSubtractedSignal();
    temp = theTkCommonModeCalculator->doIt(pedestalSubtractedSignal());
  }  else {
    cout << " Error ! No CommonModeCalculator and PedestalCalculator are defined!! " << endl; 
  }
  return temp;
}
//
//  get Zero suppressed data
//
ApvAnalysis::ZeroSuppressedSignalType TT6ApvAnalysis::zeroSuppressedSignal() {
  ApvAnalysis::ZeroSuppressedSignalType temp;
  if (theTkZeroSuppresser && theTkPedestalCalculator && 
      theTkCommonModeCalculator && theTkNoiseCalculator && theTkApvMask) {
    ApvAnalysis::RawSignalType tempraw  = (*this).rawSignal();
    ApvAnalysis::SubtractedSignalType temppsub = 
                           (*this).pedestalSubtractedSignal();
    //                          theTkPedestalCalculator->doIt(tempraw);
    ApvAnalysis::SubtractedSignalType tempcsub = 
                          theTkCommonModeCalculator->doIt(temppsub);
    temp = theTkZeroSuppresser->doIt(tempcsub, *theTkNoiseCalculator, 
                                                       *theTkApvMask);
  }else{
    cerr << "Action classes not defined!" <<endl;
  }
  return temp;
}
//
//  get Zero suppressed digis
//
//ApvAnalysis::StripDigisType TT6ApvAnalysis::zeroSuppressedDigis(){
//  ApvAnalysis::StripDigisType  temp;
//  if (theTkZeroSuppresser && theTkPedestalCalculator && 
//      theTkCommonModeCalculator && theTkNoiseCalculator && theTkApvMask) { 
//    ApvAnalysis::RawSignalType tempraw  = (*this).rawSignal();
//    ApvAnalysis::SubtractedSignalType temppsub = 
//                          (*this).pedestalSubtractedSignal();
//    //                          theTkPedestalCalculator->doIt(tempraw);
//    ApvAnalysis::SubtractedSignalType tempcsub = 
//                          theTkCommonModeCalculator->doIt(temppsub);
//    temp = theTkZeroSuppresser->doItToDigis(tempcsub, *theTkNoiseCalculator, 
//                                                         *theTkApvMask);
//  }else{
//    cerr << "Action classes not defined!" <<endl;
//  }
//  return temp;
//}
//
//  Store Pedestal Values 
//
//void TT6ApvAnalysis::storePedestals()  {
//  theApvEventReader->savePedestals(theTkPedestalCalculator->pedestal(),*this);
//}
//
//  Store Noise Values 
//
//void TT6ApvAnalysis::storeNoise()   {
//  theApvEventReader->saveNoise(theTkNoiseCalculator->stripNoise(),*this);
//}
//
//  Load Pedestal Values 
//
//void TT6ApvAnalysis::loadPedestals()   {
//  theTkPedestalCalculator->setPedestals(
//        theApvEventReader->loadPedestals(*this));
//}
//
//  Load Noise Values 
//
//void TT6ApvAnalysis::loadNoise()   {
//  theTkNoiseCalculator->setStripNoise(theApvEventReader->loadNoise(*this));
//}
//
//  Define New Event for Pedestal and Noise Calculation
//
void TT6ApvAnalysis::newEvent(){
  nEvents++;
  theRawsignalFlag = false;
  thePedSubSignalFlag = false;
  initialized = false;
  if (theTkPedestalCalculator && theTkNoiseCalculator 
                              && theTkCommonModeCalculator) {
    theTkPedestalCalculator->newEvent();
    theTkNoiseCalculator->newEvent();
    theTkCommonModeCalculator->newEvent();
  } else {
    cerr << "Error: pedestalCalculator, CommonModeCalculator or noiseCalculator missing " << endl;
  }
}
//
//  Define New Event for Pedestal and Noise Calculation
//
bool TT6ApvAnalysis::isCalibrated(){
  bool ans = false;
  if (theTkPedestalCalculator->status()->isUpdating()) ans = false;
  return false;
}

//
// Event Pre Processing
//
void TT6ApvAnalysis::updateCalibration() {
  if (initialized) return;
  initialized = true;
  thePedSubSignal = theTkPedestalCalculator->doIt(rawSignal());
  thePedSubSignalFlag = true;
  if (nEvents > nEventForInitialization) {
    if ((nEvents-nEventForInitialization)%nEventForUpdate == 1) {
      if ((nEvents - nEventForInitialization) == 1) {
	cout << "TT6ApvAnalysis: Pedestal initialization done at event " << nEvents << endl;
	ApvAnalysis::SubtractedSignalType stripRNoise = 
                theTkPedestalCalculator->rawNoise();
        theTkNoiseCalculator->setStripNoise(stripRNoise);
	theTkApvMask->calculateMask(stripRNoise);         
      } else {
	if (0) cout << "TT6ApvAnalysis: updating  at event n. " << nEvents << endl;
	ApvAnalysis::SubtractedSignalType stripNoise = theTkNoiseCalculator->stripNoise();
	theTkApvMask->calculateMask(stripNoise);
      }
   }
    //    ApvAnalysis::SubtractedSignalType temp = CMPSubtractedSignal();
    //    theTkNoiseCalculator->updateNoise(temp); 
    theTkNoiseCalculator->updateNoise(CMPSubtractedSignal()); 
  } 
}
