#include "TT6ApvFactory.h"

#include "TT6ApvMask.h"
#include "TT6ApvAnalysis.h"
#include "TT6CommonModeCalculator.h"
#include "MedianTT6CommonModeCalculator.h"
#include "ReferenceZeroSuppresser.h"
#include "ReferenceClusterZeroSuppresser.h"
#include "TkCommonMode.h"
#include "TkCommonModeTopology.h"
#include "TT6PedestalCalculator.h"
#include "TT6NoiseCalculator.h"

TT6ApvFactory* TT6ApvFactory::myInstance(0);

TT6ApvFactory* TT6ApvFactory::instance(){
  if (!myInstance) 
    myInstance = new TT6ApvFactory;
  return myInstance;
}
void TT6ApvFactory::setEventReader(ApvEventReader * in ){
  theEventReader = in;
}
ApvAnalysis* TT6ApvFactory::getApv(int in){
  if (theMap[in] == 0){
    ApvAnalysis* temp = constructApv();
    theMapReversed[temp]=in;
    theMap[in]=temp;
    cout <<" Constructed APV "<<in << " " << temp<<endl;;
  }
  return theMap[in];
}
ApvAnalysis* TT6ApvFactory::constructApv(){
  TT6ApvAnalysis* theAPV = new TT6ApvAnalysis;
  
  TT6PedestalCalculator* thePedestal = 
        new TT6PedestalCalculator(nEventForInitialization,nEventForUpdate,cutToAvoidSignal);
  TT6NoiseCalculator* theNoise = new TT6NoiseCalculator(nEventForInitialization,
                                               nEventForUpdate,cutToAvoidSignal);
  TT6ApvMask* theMask = new TT6ApvMask(fNoisyStripDefinition,
                                            cutForNoisyStrips,cutForDeadStrips);
  TkCommonModeTopology* theTopology = new TkCommonModeTopology(128,nTopology);
  TkCommonMode* theCommonMode = new TkCommonMode;
  theCommonMode->setTopology(theTopology);
  TT6CommonModeCalculator* theCM; 
  MedianTT6CommonModeCalculator* theCMMed;
  if (cmFlag != 2) {
    theCM = new TT6CommonModeCalculator(theNoise,theMask,cutToAvoidSignal);    
    theCM->setCM(theCommonMode);
  } else {
    theCMMed = new MedianTT6CommonModeCalculator(theNoise,theMask);
    theCMMed->setCM(theCommonMode);
  }
  TkZeroSuppresser* theZS = new ReferenceClusterZeroSuppresser;

  //
  // set inside the APV
  //
  theAPV->setParameters(nEventForInitialization,nEventForUpdate);
  theAPV->setZeroSuppresser(*theZS);
  theAPV->setCommonModeCalculator(*theCM);
  theAPV->setPedestalCalculator(*thePedestal);
  theAPV->setNoiseCalculator(*theNoise);
  theAPV->setMask(*theMask);
  if (theEventReader==0){
    cerr<<" ERROR: No event Reader Defined!"<<endl;
  }
  theAPV->setReader(*theEventReader);
  
  return theAPV;
}

void TT6ApvFactory::clearAllCalibrations(){
  //
  // delete of all the APVs... I know it is brutal, but it works :)
  //
  theMap.clear();
  for (map<ApvAnalysis*,int>::iterator it = theMapReversed.begin(); it!=theMapReversed.end(); it++){
    //
    // delete the Apv ands all the classes I created
    //
        deleteApv((*it).first);
  }
  theMapReversed.clear();
}

void TT6ApvFactory::deleteApv(ApvAnalysis* apv){
  cout <<"Deleting APV "<<apv<<endl;
  delete &(apv->pedestalCalculator());
  delete &(apv->noiseCalculator());
  delete &(apv->mask());
  delete &(apv->commonModeCalculator().commonMode()->topology());
  delete (apv->commonModeCalculator().commonMode());
  delete &(apv->commonModeCalculator());
  delete apv;
}
void TT6ApvFactory::setParameters(int _nEventForInitialization, int _nEventForUpdate,
                                  int _fNoisyStripDefinition, float _cutToAvoidSignal,
                                  float _cutForNoisyStrips, float _cutForDeadStrips,
                                  int _nTopology, int _cmFlag) {
  cout << "setting parameters for the TT6ApvFactory" << endl;

  cout << "nEventForInitialization  = " << _nEventForInitialization << endl;
  cout << "nEventForUpdate          = " << _nEventForUpdate << endl;
  cout << "fNoisyStripDefinition    = " << _fNoisyStripDefinition << endl;
  cout << "cutToAvoidSignal         = " << _cutToAvoidSignal << endl;
  cout << "cutForNoisyStrips        = " << _cutForNoisyStrips << endl;
  cout << "cutForDeadStrips         = " << _cutForDeadStrips << endl;
  cout << "CMTopology               = " << _nTopology << endl;
  if (_cmFlag != 2) {
    cout << " TT6CommonModeCalculator will be used " << endl;
  } else {
    cout << " MedianTT6CommonModeCalculator will be used " << endl;
  }

  nEventForInitialization  = _nEventForInitialization;
  nEventForUpdate          = _nEventForUpdate ;
  fNoisyStripDefinition    = _fNoisyStripDefinition;
  cutToAvoidSignal         = _cutToAvoidSignal ;
  cutForNoisyStrips        = _cutForNoisyStrips ;
  cutForDeadStrips         = _cutForDeadStrips ;
  if(_nTopology >= 128 || _nTopology <0) {
    nTopology              =  128;
  } else {
    nTopology              = _nTopology;
  } 
  cmFlag                   = _cmFlag; 
}
