#include "TT6CommonModeCalculator.h"
#include <cmath>
#include <numeric>
#include <algorithm>
// 
//  Constructor :
//
TT6CommonModeCalculator::TT6CommonModeCalculator(TkNoiseCalculator* noise_calc,
			TkApvMask* mask_calc, float sig_cut) : 
  theNoiseCalculator(noise_calc),
  theApvMask(mask_calc),
  cutToAvoidSignal(sig_cut),
  alreadyUsedEvent(false)
{
  if (0) cout << "Constructing TT6CommonMode Calculator ..." << endl;
}
//
//  Destructor 
//
TT6CommonModeCalculator::~TT6CommonModeCalculator() {
  if (0) cout << "Destructing TT6CommonModeCalculator " << endl;
}
//
// Action :
//
ApvAnalysis::SubtractedSignalType TT6CommonModeCalculator::doIt
                (ApvAnalysis::SubtractedSignalType indat) 
{
  ApvAnalysis::SubtractedSignalType out;
  calculateTruncatedCommonMode(indat);
  
  int setNumber;
  for (unsigned int i=0; i<indat.size(); i++){
    setNumber = theTkCommonMode->topology().setOfStrip(i);
    out.push_back(indat[i] - theCommonModeValues[setNumber]);
    //    out.push_back(indat[i] - theCommonModeValues[0]);
  }  
  return out;
}
//
//  Calculation of Common Mode Values :
//
void TT6CommonModeCalculator::calculateCommonMode
        (ApvAnalysis::SubtractedSignalType& indat) 
{ 
  if (alreadyUsedEvent == false) {
    alreadyUsedEvent = true;

    TkApvMask::MaskType strip_mask = theApvMask->mask();
    TkNoiseCalculator::NoiseType strip_noise = theNoiseCalculator->stripNoise();

    int nSet = theTkCommonMode->topology().numberOfSets();
    theCommonModeValues.clear();

    for (int i=0; i<nSet; i++){
      int initial   = theTkCommonMode->topology().initialStrips()[i];
      int final     = theTkCommonMode->topology().finalStrips()[i];
      //      int initial   = 0;
      //      int final     = 127;
      float sumVal = 0.0;
      float sumWt =  0.0;
      for (int j = initial; j <= final; j++) {
        if (strip_mask[j] == TkApvMask::ok && 
              fabs(indat[j]) < cutToAvoidSignal*strip_noise[j]) { 
	  //	  float val = indat[j];
	  float nWeight = 1/(strip_noise[j]*strip_noise[j]);
	  sumVal += (indat[j]*nWeight);
	  sumWt  += nWeight;
        }
      }
      double avVal = (sumWt) ? sumVal/sumWt :0.0;
      theCommonModeValues.push_back(static_cast<float>(avVal));
    }
    TT6CommonModeCalculator::setCM(theCommonModeValues);
  }
}
//
//  Calculation of Common Mode Value using truncated average
//
void TT6CommonModeCalculator::calculateTruncatedCommonMode
        (ApvAnalysis::SubtractedSignalType& indat) 
{ 
  if (alreadyUsedEvent == false) {
    alreadyUsedEvent = true;

    TkApvMask::MaskType strip_mask = theApvMask->mask();

    ApvAnalysis::SubtractedSignalType temp_indat;
    int nSet = theTkCommonMode->topology().numberOfSets();
    theCommonModeValues.clear();

    for (int i=0; i<nSet; i++){
      int initial   = theTkCommonMode->topology().initialStrips()[i];
      int final     = theTkCommonMode->topology().finalStrips()[i];
      //      int initial   = 0;
      //      int final     = 127;
      // copy good strips in a temporary vector
      for (int j = initial; j <= final; j++) {
        if (strip_mask[j] == TkApvMask::ok) {
          temp_indat.push_back(indat[j]);  
        }
      }         
      sort(temp_indat.begin(), temp_indat.end());

      int len     = temp_indat.size();
      int nTruncL = int(len*0.20);
      int nTruncH = int(len*0.20);
      int rlen    = len - nTruncL - nTruncH;

      double sumVal = 0.0;
      sumVal   = accumulate((temp_indat.begin()+nTruncL), 
                            (temp_indat.end()-nTruncH), sumVal);
      double avVal = (rlen) ? sumVal/rlen   : 0.0;
         
      theCommonModeValues.push_back(static_cast<float>(avVal));
    }
    TT6CommonModeCalculator::setCM(theCommonModeValues);
  }
}
//
// Define New Event
// 
void TT6CommonModeCalculator::newEvent() {
  alreadyUsedEvent = false;
}

