#include "TT6StripChargeFinder.h"
#include <cmath>
#include <numeric>
#include <algorithm>
//
// -- Constructor.
//
TT6StripChargeFinder::TT6StripChargeFinder(int ntrH, 
           int ntrL) : nTruncInHigher(ntrH), 
                       nTruncInLower(ntrL){
  cout << "Constructing TT6StripChargeFinder " << endl;
}
//
//  -- Destructor  
//
TT6StripChargeFinder::~TT6StripChargeFinder() {
  cout << "Destructing TT6StripChargeFinder " << endl;
}
//
// -- Find Strip Charges 
//
ApvAnalysis::SubtractedSignalType TT6StripChargeFinder::doIt(
          ApvAnalysis::SubtractedSignalType in_data, 
	  TkApvMask::MaskType& in_mask) {

  /* Calculate Common Mode */
  ApvAnalysis::SubtractedSignalType temp_data;

  for (unsigned int j = 0; j < in_data.size(); j++) {
    if (in_mask[j] == TkApvMask::ok) {
      temp_data.push_back(in_data[j]);  
    }
  }           
  sort(temp_data.begin(), temp_data.end());
  int len = temp_data.size();
  int rlen = len - nTruncInHigher - nTruncInLower;
  double sumVal = 0.0;
  sumVal   = accumulate((temp_data.begin()+nTruncInLower), 
		 (temp_data.end()-nTruncInHigher),sumVal);
  double avVal = (rlen) ? sumVal/rlen   : 0.0;
  /* Subtract Common Mode Now */
  ApvAnalysis::SubtractedSignalType cmsub_data;
  for (unsigned int i = 0; i < in_data.size(); i++) {
    cmsub_data.push_back(in_data[i]-avVal);  
  }
  return cmsub_data;
}
