#ifndef VIEWER_ControlSettingTab
#define VIEWER_ControlSettingTab

#include <string>
#ifndef ROOT_TGFrame
#include "TGFrame.h"
#endif
#include "TGButton.h"
#include "TGLabel.h"
#include "TGTextView.h"
#include "TGProgressBar.h"
#include "TGLabeledEntry.h"

#include <pthread.h> // controls the Thread mechanism
//#include "Histogrammer.h"

#define  NCHANMAX 6
#define  NSTRIP 128



enum EControlSettingTabId {
  /* Analysis Type Radio Button */
  RB_ANAL_A,
  RB_ANAL_B,  
  RB_ANAL_C,  
  RB_ANAL_D,

  CB_RDATA_SAVE,
   
  TB_ANAL_CONFIG,
  TB_DAQ_START,
  TB_DAQ_STOP,

  TB_HISTO_SAVE,
  TB_HISTO_CLEAR
};
void * takeRawDataSim(void *ptr ); // should be general data member 
void * takeRawData(void *ptr ); // should be general data member 
void * takeCalibData(void *ptr ); // should be general data member 
class ControlSettingTab : public TGCompositeFrame 
{
  //RQ_OBJECT("ControlSettingTab") 

public:
   ControlSettingTab(const TGMainFrame *parent, const TGWindow *main, 
                UInt_t w, UInt_t h, UInt_t options = 0, 
                ULong_t back = fgDefaultFrameBackground);
   ~ControlSettingTab();
   void       CreateDisplay();
   Bool_t     ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2);
   void       startDataTaking(); 
   void       stopDataTaking();
   void       saveHistograms();
   void       clearHistograms();
   
   int        getEventsToBeProcessed();
   int        getProcessType();
   int        getState();
   string     getRawDataFileName();
   int        getInitEvent();
   int        getUpdateEvent();

   TGHProgressBar* getProgressBar();
   void addHistoToCanvas(int ich, int istr);

   void       readAndLoadConfig(string fnake_config);
   void       doItAutoRun(string fnake_config, int data_type);

private:
   pthread_t aThread;
   void       setRawDataFileName();

   //
   // Analysis Type
   //
   const TGMainFrame*  fParent; // cast from ModuleTestApp
   void             PedestalsHisto();
 
   TGRadioButton*   fAnalTypeRB[4];
   TGLabeledEntry*  fAnalEventEntry[4];
   TGGroupFrame*    fAnalTypeFrame;
   TGGroupFrame*    fAnalEventFrame;
   TGCheckButton*   fSaveRDataCB;
   TGTextButton*    fAnalConfigButton;
   TGVerticalFrame* fAnalConfigFrame;
   TGGroupFrame*    fAnalGFrame;
   
   TGTextButton*    fStartButton;
   TGTextButton*    fStopButton;
   TGHProgressBar*  fProgressBar;
   TGGroupFrame*    fRunControlFrame;

   TGTextButton*    fSaveHistoButton;
   TGTextButton*    fClearHistoButton;
   TGGroupFrame*    fStorageFrame;
       
   TGLayoutHints*   fL;
   TGLayoutHints*   fLAnal;
   TGLayoutHints*   fLNum;

   int              fProcessFlag;
   int              fDataTakingState;
   bool             fRawDataToBeWritten;
   bool             fAutoRunFlag;

   // Analysis Parameters
   int              eventsForInitialisation;
   int              eventsForUpdate;
   int              noisyStripDefinition;
   float            cutToAvoidSignal;
   float            cutForNoisyStrips;
   float            cutForDeadStrips;
   int              nTopology;
   int              cmFlag;

   string           fRawDataFileName;

};

#endif
