#ifndef VIEWER_HVSettingTab
#define VIEWER_HVSettingTab

#ifndef ROOT_TGFrame
#include "TGFrame.h"
#endif
#include "TGButton.h"
#include "TGLabel.h"
#include "TGNumberEntry.h"
#include "TGComboBox.h"
#include "TGTextView.h"
#include "TGProgressBar.h"
#include "TTimer.h"

#include<string>
#include<map>

class HVChannelInterface;

enum EHVSettingTabId {  

  CM_CRATE_TYPE,

  TB_HV_REQ,
  TB_HV_REL,

  TB_HV_SREAD,
  TB_HV_SET,
  TB_HV_AREAD,
  TB_HV_TOG,
  TB_HV_AREL,

  NE_HV_DEVICE,
  NE_HV_BOARD,
  NE_HV_CHAN,
  NE_HV_VOLT,
  NE_HV_CURR,
  NE_HV_RUP,
  NE_HV_RDOWN,
  NE_HV_VMON,
  NE_HV_IMON

};
class HVSettingTab : public TGCompositeFrame {

public:
   HVSettingTab(const TGMainFrame *parent, const TGWindow *main, 
                UInt_t w, UInt_t h, UInt_t options = 0, 
                ULong_t back = fgDefaultFrameBackground);
   ~HVSettingTab();
   void       CreateDisplay();
   Bool_t     ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2);
   void       requestChannel();
   void       requestSY127Channel();
   void       releaseSingleHVChannel();
   void       releaseAllHVChannels(int iflag);
   void       readSingleHVSetting();
   void       readAllHVSettings();
   void       applyHVSetting();
   void       toggleHVSetting();
   void       monitorHVStatus();

   void       readAndLoadConfig(string fname_config);
   void       doItAutoRun(string fname_config);

   ClassDef(HVSettingTab,0)
private:

   int       readChannelIdentity();


   TGComboBox*           fHVCrateTypeBox;
   TGNumberEntry*        fHVCrateNumNE;
   TGNumberEntry*        fHVBoardNumNE;
   TGNumberEntry*        fHVChannelNumNE;
   TGLabel*              fHVCrateTypeLab;
   TGLabel*              fHVCrateNumLab;
   TGLabel*              fHVBoardNumLab;
   TGLabel*              fHVChannelNumLab;
   TGTextButton*         fHVRequestBut;
   TGTextButton*         fHVReleaseBut;
   TGGroupFrame*         fHVDevGFrame;


   TGNumberEntry*        fVoltageNE;
   TGNumberEntry*        fCurrentNE;
   TGNumberEntry*        fRampUpNE;
   TGNumberEntry*        fRampDownNE;
   TGHProgressBar*       fVMonitorBar;
   TGHProgressBar*       fIMonitorBar;
   TGLabel*              fVoltageLab;
   TGLabel*              fCurrentLab;
   TGLabel*              fRampUpLab;
   TGLabel*              fRampDownLab;
   TGLabel*              fVMonitorLab;
   TGLabel*              fIMonitorLab;
   TGGroupFrame*         fSettingGFrame;

   TGTextButton*         fHVReadBut;
   TGTextButton*         fHVSetBut;
   TGTextButton*         fHVReadAllBut;   
   TGTextButton*         fHVToggleBut;
   TGTextButton*         fReleaseAllBut;   
    
   TGGroupFrame*         fActionGFrame;  

   TGHorizontalFrame*    fHVMessageFrame;
   TGTextView*           fHVMessageView;
   
   TGLayoutHints*        fL;
   TGLayoutHints*        fLLab;
   TGLayoutHints*        fLBut;

   TTimer*               fMonitorTimer;
   
   int                   fHVCrateType;
   int                   fHVCrateNum, fHVBoardNum, fHVChannelNum;

   bool                  fOn;
   HVChannelInterface*   fHVChannel;
   map <int, HVChannelInterface*>   fHVChannelMap;

};

#endif
