
#ifndef VIEWER_ModuleTestApp
#define VIEWER_ModuleTestApp

/////////////////////////////////////////////////////////////////////
//
//  Application specific declarations. ModuleTestApp is the main window
//  for the viewer application which contains a menubar, a number of  
//  tabs, status bars etc. The tabs in turn contain the detail
//  of the application. 
//
//  
//
/////////////////////////////////////////////////////////////////////

#ifndef ROOT_TGFrame
#include "TGFrame.h"
#endif
#include "TGTab.h"
#include "TGMenu.h"
#include "TGButton.h"
#include "TGLabel.h"
#include "TTimer.h"

#include "HVSettingTab.h"
#include "ControlSettingTab.h"
#include "HistogramViewingTab.h"
#include "OnlineHistogramTab.h"

#include<string>
enum EModuleTestAppId {

  TB_MAIN_CONF_SAVE,
  TB_MAIN_XML_SAVE,
  TB_MAIN_AUTO_START,
  TB_MAIN_REL,
  TB_MAIN_HELP,
  TB_MAIN_RELX
};
class ModuleTestApp : public TGMainFrame {
public:
  ModuleTestApp(const TGWindow *p, UInt_t w, UInt_t h, UInt_t options);
  virtual ~ModuleTestApp();
  void     addTabs();
  void     CreateDisplay();
  TGTab    *getTab();
  Bool_t   ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2);
  void     saveConfiguration();
  void     doItXMLDBase();
  void     doItAutoRun();
  void     releaseChannels();
  void     releaseAndExit();
  int      getCurrentTabId();
  void     CloseWindow();
  void     showHelp();

  HVSettingTab*        getHVSettingTab();
  ControlSettingTab*   getControlSettingTab();
  HistogramViewingTab* getHistogramViewingTab();
  OnlineHistogramTab*  getOnlineHistogramTab();

  void     setLoclDBFile(string fname);
  void     setOperatorName(string oname);
  void     setRunConfigFile(string fname);
  void     setSaveRootFile(string fname);
  string   getSaveRootFile();
protected:
  /* General variables */
  TGLayoutHints *fGeneralLayout;


  /*  Tab Widget related variables */

  TGTextButton*       fSaveConfBut;
  TGTextButton*       fSaveXMLBut;
  TGTextButton*       fAutoStartBut;
  TGTextButton*       fReleaseBut;
  TGTextButton*       fHelpBut;
  TGTextButton*       fReleaseExitBut;
  TGCompositeFrame*   fButtonFrame;
  TGLayoutHints*      fL;

  TGCompositeFrame*    fMainTabFrame;
  TGTab*               fMainTab;
  TGCompositeFrame*    tf;

  HVSettingTab*        fHVSettingTab;
  ControlSettingTab*   fControlSettingTab;
  HistogramViewingTab* fHistogramViewingTab;
  OnlineHistogramTab*  fOnlineHistogramTab;

  TTimer*              fRawTimer;
  TTimer*              fPedTimer;
  TTimer*              fRNoiseTimer;
  TTimer*              fCMNoiseTimer;
  TTimer*              fCMTimer;
  TTimer*              fCalibTimer;

  string fLocalDBFile;
  string fOperatorName;
  string fRunConfigFileName;
  string fSaveRootFileName;
};
#endif

