//
//  The tab consists of APV, PLL and MUX  Settings
//
#include <iostream.h>
#include <strstream.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h> // controls the:  time_t ...
#include <vector>
#include <algorithm>
#include <numeric>

#include "TROOT.h"
#include "TSystem.h"
#include "TKey.h"
#include "TGMsgBox.h"
#include "TCanvas.h"
#include "TRandom.h"
#include "TStopwatch.h"
#include "TGFileDialog.h"
#include "TFile.h"

#include "ControlSettingTab.h" 
#include "ModuleTestApp.h"
#include "Histogrammer.h"
#include "HistosForAPV.h"

#include "ApvAnalysis.h"
#include "TkCommonModeCalculator.h"
#include "TT6ApvFactory.h"
#include "TT6EventReader.h"
#include "TT6NoiseCalculator.h"
#include "TT6StripChargeFinder.h"
#include "TT6Timer.h"
//
// Constructor  
//
 ControlSettingTab::ControlSettingTab(const TGMainFrame *parent, const TGWindow* main, UInt_t w, UInt_t h, UInt_t options, ULong_t back) 
  : TGCompositeFrame(main, w, h, options, back)
    //    fHistogrammer(0)
{
  fParent = parent;
  CreateDisplay();

  fProcessFlag = -1;
  fDataTakingState = 0;
  fRawDataToBeWritten  = false;
  fAutoRunFlag  = false;    
  
  // Default Values for Analysis Parameters
  eventsForInitialisation = 500;
  eventsForUpdate         = 100;
  noisyStripDefinition    = 1;
  cutToAvoidSignal        = 3.0;
  cutForNoisyStrips       = 5.0;
  cutForDeadStrips        = 0.5;
  nTopology               = 128;
  cmFlag                  = 1;
}
// 
// Destructor 
//
ControlSettingTab::~ControlSettingTab() 
{
  
  // Delete Analysis Settings 
   
  for (int i = 0; i < 4; i++) {
    delete    fAnalTypeRB[i];
    delete    fAnalEventEntry[i];
  }
  delete    fSaveRDataCB;
  delete    fAnalConfigButton;
  delete    fAnalTypeFrame;
  delete    fAnalEventFrame;
  delete    fAnalConfigFrame;
  delete    fAnalGFrame;

  // Delete Run Control Settings  
  delete    fStartButton;
  delete    fStopButton;
  delete    fProgressBar;
  delete    fRunControlFrame;

  // Delete Storage Setting  
  delete    fSaveHistoButton;
  delete    fClearHistoButton;
  delete    fStorageFrame;

  delete    fL; 
  delete    fLAnal;
  delete    fLNum;

  //  if (fHistogrammer) delete fHistogrammer;
}
void ControlSettingTab::CreateDisplay() 
{
  fLAnal = new TGLayoutHints(kLHintsTop|kLHintsLeft,20,20,20,20);
  fLNum  = new TGLayoutHints(kLHintsLeft|kLHintsTop,18,18,18,18);
  fL     = new TGLayoutHints(kLHintsTop | kLHintsLeft,10, 10, 10, 2);
  
  // Analysis Type and Number of Events  

  fAnalGFrame = new TGGroupFrame(this, " Analysis Setting ", 
				 kHorizontalFrame);
  AddFrame(fAnalGFrame);
  // Analysis Types  
  fAnalTypeFrame = new TGGroupFrame(fAnalGFrame, " Event Type ", 
				    kVerticalFrame);
  fAnalGFrame->AddFrame(fAnalTypeFrame, fLAnal);

  fAnalTypeRB[0] = new TGRadioButton(fAnalTypeFrame, 
				 new TGHotString(" Raw "),  RB_ANAL_A);
  fAnalTypeRB[1] = new TGRadioButton(fAnalTypeFrame, 
				 new TGHotString(" Pedestal "),  RB_ANAL_B);
  fAnalTypeRB[2] = new TGRadioButton(fAnalTypeFrame, 
				 new TGHotString(" Ped + Calib "),  RB_ANAL_C);
  fAnalTypeRB[3] = new TGRadioButton(fAnalTypeFrame, 
				 new TGHotString(" Ped + LASER "),  RB_ANAL_D);
  for (int i = 0; i < 4; i++) {
    fAnalTypeFrame->AddFrame(fAnalTypeRB[i], fLAnal);
    fAnalTypeRB[i]->Associate(this);
  }
  // Events to be Analysed   
  fAnalEventFrame = new TGGroupFrame(fAnalGFrame, 
				     " # of Events to take ", kVerticalFrame);
  fAnalGFrame->AddFrame(fAnalEventFrame, fLAnal); 
  for (int k = 0; k < 4; k++) {
    fAnalEventEntry[k] = new TGLabeledEntry(fAnalEventFrame, " ", 150, 20, 5);
    fAnalEventFrame->AddFrame(fAnalEventEntry[k], fLNum);
  }
  // Analysis Parameters and Option to Save Raw Data in Ascii file 
  fAnalConfigFrame = new TGVerticalFrame(fAnalGFrame, 10, 10);
  fAnalGFrame->AddFrame(fAnalConfigFrame);  
  
  fSaveRDataCB = new TGCheckButton(fAnalConfigFrame,"Save Raw Data",
                                                           CB_RDATA_SAVE);
  fSaveRDataCB->Associate(this);
  fAnalConfigFrame->AddFrame(fSaveRDataCB, fLAnal);  

  fAnalConfigButton = new TGTextButton(fAnalConfigFrame, 
                       "&Load Configuration", TB_ANAL_CONFIG);
  fAnalConfigButton->Resize(150,20);
  fAnalConfigButton->Associate(this);
  fAnalConfigButton->SetToolTipText("Read and Load Configuration");
  fAnalConfigFrame->AddFrame(fAnalConfigButton, fLAnal);
  // Run Control Setting  
 
  fRunControlFrame = new TGGroupFrame(this, " Run Control Settings ", 
				      kHorizontalFrame);
  AddFrame(fRunControlFrame);
  
  fStartButton = new TGTextButton(fRunControlFrame,"&Start",TB_DAQ_START);
  fStartButton->Resize(150,20);
  fStartButton->Associate(this);
  fStartButton->SetToolTipText("Start Data Taking");
  fRunControlFrame->AddFrame(fStartButton, fL);

  fStopButton = new TGTextButton(fRunControlFrame, "&Stop",TB_DAQ_STOP);
  fStopButton->Resize(150,20);
  fStopButton->Associate(this);
  fStopButton->SetToolTipText("Stop Data Taking");
  fRunControlFrame->AddFrame(fStopButton, fL);

  fProgressBar = new TGHProgressBar(fRunControlFrame, TGProgressBar::kFancy,300);
  fProgressBar->SetFillType(TGProgressBar::kBlockFill);
  fProgressBar->SetBarColor("lightblue");
  fProgressBar->ShowPosition(kTRUE, kTRUE, "%.0f Events");
  fRunControlFrame->AddFrame(fProgressBar,fL);

  // Data and Histogram Storage Setting  
  fStorageFrame = new TGGroupFrame(this, " Storage", kVerticalFrame);
  AddFrame(fStorageFrame);
  
  fSaveHistoButton = new TGTextButton(fStorageFrame, "&Save Histograms ",TB_HISTO_SAVE);
  fSaveHistoButton->Associate(this);
  fSaveHistoButton->Resize(200,20);
  fSaveHistoButton->SetToolTipText("Save Histograms");
  fStorageFrame->AddFrame(fSaveHistoButton, fL);
  fSaveHistoButton->SetState(kButtonDisabled);

  fClearHistoButton = new TGTextButton(fStorageFrame, "&Clear Histograms",TB_HISTO_CLEAR);
  fClearHistoButton->Associate(this);
  fClearHistoButton->Resize(150,20);
  fClearHistoButton->SetToolTipText("Clear Booked Histograms");
  fStorageFrame->AddFrame(fClearHistoButton, fL);
  fClearHistoButton->SetState(kButtonDisabled);
}
// 
// Handle user actions 
//
Bool_t ControlSettingTab::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2) 
{
  // Handle messages
  switch (GET_MSG(msg)) {

  case kC_COMMAND:
    switch (GET_SUBMSG(msg)) {

    case kCM_RADIOBUTTON:
      switch (parm1) {

      case RB_ANAL_A:
	{
	  fProcessFlag = 1;
          fSaveRDataCB->SetState(kButtonDisabled);
	  fAnalEventEntry[0]->getTGTextEntry()->SetState(kTRUE);
	  fAnalTypeRB[1]->SetState(kButtonUp);
	  fAnalEventEntry[1]->getTGTextEntry()->SetState(kFALSE);
	  fAnalTypeRB[2]->SetState(kButtonUp);
	  fAnalEventEntry[2]->getTGTextEntry()->SetState(kFALSE);
	  fAnalTypeRB[3]->SetState(kButtonUp);
	  fAnalEventEntry[3]->getTGTextEntry()->SetState(kFALSE);
	  break;
        }
      case RB_ANAL_B:
	{ 
          fProcessFlag = 2;
          fSaveRDataCB->SetState(kButtonUp);
   	  fAnalTypeRB[0]->SetState(kButtonUp);
	  fAnalEventEntry[0]->getTGTextEntry()->SetState(kFALSE);
	  fAnalEventEntry[1]->getTGTextEntry()->SetState(kTRUE);
	  fAnalTypeRB[2]->SetState(kButtonUp);
	  fAnalEventEntry[2]->getTGTextEntry()->SetState(kFALSE);
	  fAnalTypeRB[3]->SetState(kButtonUp);
	  fAnalEventEntry[3]->getTGTextEntry()->SetState(kFALSE);
	  break;
        }
      case RB_ANAL_C:
	{  
          fProcessFlag = 3;
          fSaveRDataCB->SetState(kButtonDisabled);
	  fAnalTypeRB[0]->SetState(kButtonUp);
	  fAnalEventEntry[0]->getTGTextEntry()->SetState(kFALSE);
	  fAnalTypeRB[1]->SetState(kButtonUp);
	  fAnalEventEntry[1]->getTGTextEntry()->SetState(kFALSE);
	  fAnalEventEntry[2]->getTGTextEntry()->SetState(kTRUE);
	  fAnalTypeRB[3]->SetState(kButtonUp);
	  fAnalEventEntry[3]->getTGTextEntry()->SetState(kFALSE);
	  break;
        }
      case RB_ANAL_D:  
   	{ 
          fProcessFlag = 4;
          fSaveRDataCB->SetState(kButtonDisabled);
	  fAnalTypeRB[0]->SetState(kButtonUp);
	  fAnalEventEntry[0]->getTGTextEntry()->SetState(kFALSE);
	  fAnalTypeRB[1]->SetState(kButtonUp);
	  fAnalEventEntry[1]->getTGTextEntry()->SetState(kFALSE);
	  fAnalTypeRB[2]->SetState(kButtonUp);
	  fAnalEventEntry[2]->getTGTextEntry()->SetState(kFALSE);
	  fAnalEventEntry[3]->getTGTextEntry()->SetState(kTRUE);
	  break;
        }
      default:
       break;
     
      }
    case kCM_BUTTON:
      switch (parm1) {

      case TB_ANAL_CONFIG:        
        {
          readAndLoadConfig(" ");
          break;
        }
      case TB_DAQ_START:
        { 
          if (fProcessFlag == -1 ) {
            int retval;
            new TGMsgBox(fClient->GetRoot(), this, "Run Control Message",
	       "Please Select Type & #of Events and then Try !!",
                kMBIconExclamation, kMBOk, &retval);  
	  } else { 
	    //	    fDataTakingState = 1;
	    //	    fProgressBar->Reset();
	    //      fSaveHistoButton->SetState(kButtonUp);
	    //      fClearHistoButton->SetState(kButtonUp);
	    //      fStartButton->SetState(kButtonDisabled);
	    startDataTaking();
          }
          break;
        }
      case TB_DAQ_STOP:
	{
	  //      fDataTakingState = 2;
	  //	  fProgressBar->Reset();
	  //      fSaveHistoButton->SetState(kButtonUp);
	  //      fClearHistoButton->SetState(kButtonUp);        
          stopDataTaking();
	  break;
        }
      case TB_HISTO_SAVE:
        { 
  	  saveHistograms();
          fSaveHistoButton->SetState(kButtonDisabled);
	  break;
        }

      case TB_HISTO_CLEAR:
        {
	  clearHistograms();
          fClearHistoButton->SetState(kButtonDisabled);
	  break;
        }

      default:
	break;
      }
    }
  }
  return kTRUE;
}
// 
// Start Data Taking
//
void  ControlSettingTab::startDataTaking() {

  fDataTakingState = 1;
  fProgressBar->Reset();
  fStartButton->SetState(kButtonDisabled);
  
  // Raw Data Saving in Ascii File
  if (fSaveRDataCB->GetState() == kButtonDown && fProcessFlag == 2) {
    fRawDataToBeWritten = true;
    setRawDataFileName();
  }
  // Set Analysis Parameters in the TT6ApvFactory
  TT6ApvFactory::instance()->setParameters(eventsForInitialisation, 
                                           eventsForUpdate,
                                           noisyStripDefinition,
                                           cutToAvoidSignal,
                                           cutForNoisyStrips,
                                           cutForDeadStrips,
                                           nTopology,
                                           cmFlag);

  // Create Histogrammer  ONLY ONCE 
  int nFedChan = 4;

  Histogrammer* fHistogrammer = Histogrammer::Instance();
  fHistogrammer->clearAllHistograms();
  for (int i = 0; i < nFedChan; i++) {
    fHistogrammer->addAPVHistos(new HistosForAPV(i));
  }  
  // Book Histograms  
  if (fProcessFlag == 1) {
    fHistogrammer->bookRawHistograms();
  } else if (fProcessFlag == 2) {
    fHistogrammer->bookPedHistograms();
  } else if (fProcessFlag == 3) {
    fHistogrammer->bookCalibHistograms();
  }
  addHistoToCanvas(1,1);
  // Create a thread 
  int threadStatus;
  threadStatus = pthread_create(&aThread, NULL,takeRawDataSim,(void *) fParent);
  if (threadStatus != 0) {
    cout << "Thread creating failed" << endl;
  } else {
    cout << "Thread created" << endl;
  }
}
// 
// Stop Data Taking
//
void ControlSettingTab::stopDataTaking() {
  fDataTakingState = 2;
  int threadStat = pthread_join(aThread, NULL);
  if(threadStat == 0) {
    cout << "Thread joined succesfully and Data State = " << fDataTakingState  << endl;
  }
  cout <<" CLEARING ALL THE CALIBRATIONS! (deleting APVs)"<<endl;
  TT6ApvFactory::instance()->clearAllCalibrations();
  fDataTakingState = 0;
  fProgressBar->Reset();

  fStartButton->SetState(kButtonUp); 
  fSaveHistoButton->SetState(kButtonUp);
  fClearHistoButton->SetState(kButtonUp);       
  fSaveRDataCB->SetState(kButtonUp);
  fRawDataToBeWritten = false;

  // Save Histograms If it is "Automatic Run"
  if (fAutoRunFlag) {
    this->saveHistograms();
    fSaveHistoButton->SetState(kButtonDisabled);
    fAutoRunFlag = false;
  } 
}
// 
//  -- Save Histograms
//
void ControlSettingTab::saveHistograms() {
  const char* fname_root;
  string file_name;
  if (!fAutoRunFlag) {
    TGFileInfo fi;
    const char *fRootFileTypes[] = {"Root Files",    "*.root"};
    fi.fFileTypes = fRootFileTypes;
    new TGFileDialog(fClient->GetRoot(), this, kFDSave, &fi);

    if (!fi.fFilename) {
      int retval;
      new TGMsgBox(fClient->GetRoot(), this, "Run Control Message",
	  " Error !! File Cannot be Opened !! ",
           kMBIconExclamation, kMBOk, &retval);  
      return;
    } else {
       fname_root = fi.fFilename;
    }
  } else {
    fname_root = ((ModuleTestApp*) fParent)->getSaveRootFile().c_str();
  }
  cout << " Saving Histograms in " << fname_root << endl;
  TFile *fRootFile = new TFile(fname_root,"RECREATE");
  Histogrammer* fHistogrammer = Histogrammer::Instance();
  fHistogrammer->saveAllHistograms(fProcessFlag);
  fRootFile->Write();  
}
// 
//  -- Clear Histograms
//
void ControlSettingTab::clearHistograms()
{
  cout << " Clear Histograms " << endl;
  Histogrammer* fHistogrammer = Histogrammer::Instance();
  fHistogrammer->clearAllHistograms();
}
// 
//   --Set Raw Data File Name
//
void ControlSettingTab::setRawDataFileName()
{
  static char def_name[] = "rawdata.dat";
  TGFileInfo fi;
  
  new TGFileDialog(fClient->GetRoot(), this, kFDOpen, &fi);
  if (!fi.fFilename) {
    int retval;
    new TGMsgBox(fClient->GetRoot(), this, "Run Control Message",
    "File with specified name Cannot be Opened using default name!!",
     kMBIconExclamation, kMBApply, &retval);  
    fRawDataFileName = def_name;
  } else {
    fRawDataFileName = fi.fFilename;
  } 
  cout << "File Name For Raw Data " <<  fRawDataFileName << endl;
}
//
//  -- Get Number of Events to be Processed
// 
inline int  ControlSettingTab::getEventsToBeProcessed()
{
  int nEvents =  atoi(fAnalEventEntry[fProcessFlag-1]->GetText());
  return nEvents;
}
//
//  -- Get State of the Data Taking
//
inline  int ControlSettingTab::getState()
{
  return  fDataTakingState;
}
//
//  -- Get Process Type (Raw, Ped, Cal, Laser)
//
inline  int ControlSettingTab::getProcessType()
{
  return  fProcessFlag;
}
//
//  -- Get Progress Bar 
//
inline  TGHProgressBar* ControlSettingTab::getProgressBar()
{
  return  fProgressBar;
}
//
//  -- Get Raw Data File Name
//
inline string ControlSettingTab::getRawDataFileName() 
{
  if (fRawDataToBeWritten) {
    cout << " Raw Data To be Written in " << fRawDataFileName << endl;
    return fRawDataFileName;
  } else {
    cout << " Raw Data Not To be Written !!! " << endl;
    return "";
  }
}
//
//  -- Get Events for Initialisation
//
inline int ControlSettingTab::getInitEvent() {
  return eventsForInitialisation;
}
//
//  -- Get Events for Update
//
inline int ControlSettingTab::getUpdateEvent() {
  return eventsForUpdate;
}
//
//  -- Add a particular Histogram to the canvas
//
void ControlSettingTab::addHistoToCanvas(int ich, int istr)
{
  Histogrammer* fHistogrammer = Histogrammer::Instance();  
  TH1F* hr = fHistogrammer->getRawHistogram(ich-1);
  if (hr != 0) {
    TCanvas* fTRCanvas = ((ModuleTestApp*) fParent)->getOnlineHistogramTab()->getRawCanvas();
    fTRCanvas->Divide(1,1);
    fTRCanvas->Clear();
    fTRCanvas->cd(1);
    hr->Draw(); 
  }
  if (fProcessFlag  > 1) {

    TProfile* hp = fHistogrammer->getPedHistogram(ich-1);
    if (hp != 0) {
      TCanvas* fTPCanvas = ((ModuleTestApp*) fParent)->getOnlineHistogramTab()->getPedCanvas();
      fTPCanvas->Divide(1,1);
      fTPCanvas->Clear();
      fTPCanvas->cd(1);
      hp->Draw(); 
    }    
    TH1F* hrn = fHistogrammer->getRNoiseHistogram(ich-1);
    if (hrn != 0) {
      TCanvas* fTNCanvas = ((ModuleTestApp*) fParent)->getOnlineHistogramTab()->getRNoiseCanvas();
      fTNCanvas->Divide(1,1);
      fTNCanvas->Clear();
      fTNCanvas->cd(1);
      hrn->Draw(); 
    }    
    TH1F* hn = fHistogrammer->getCMNoiseHistogram(ich-1);
    if (hn != 0) {
      TCanvas* fTNCanvas = ((ModuleTestApp*) fParent)->getOnlineHistogramTab()->getCMNoiseCanvas();
      fTNCanvas->Divide(1,1);
      fTNCanvas->Clear();
      fTNCanvas->cd(1);
      hn->Draw(); 
    }    
    TH1F* hcm = fHistogrammer->getCMHistogram(ich-1);
    if (hcm != 0) {
      TCanvas* fTCCanvas = ((ModuleTestApp*) fParent)->getOnlineHistogramTab()->getCMCanvas();
      fTCCanvas->Divide(1,1);
      fTCCanvas->Clear();
      fTCCanvas->cd(1);
      hcm->Draw(); 
    }    
  }
  if (fProcessFlag == 3) {
    TH1F* hcal = fHistogrammer->getCalibHistogram(ich-1, istr-1);
    if (hcal != 0) {
      TCanvas* fTClCanvas = ((ModuleTestApp*) fParent)->getOnlineHistogramTab()->getCalibCanvas();
      fTClCanvas->Divide(1,1);
      fTClCanvas->Clear();
      fTClCanvas->cd(1);
      hcal->Draw(); 
    }    
  }
}
//
// -- Read and Load Analysis Parameters from Configuration File
//
void ControlSettingTab::readAndLoadConfig(string fname_config){
  cout << " Not Yet Implemented " << endl;
}
//
// -- Actions for Automatic Running
// 
void ControlSettingTab::doItAutoRun(string fname_config, int data_type) {
  cout << " Not Yet Implemented " << endl;
}  
//
// Take Simulated Raw Data
//
void * takeRawDataSim(void *ptr) {
    
  int fNtotal;
  int fNSpill, fNAPVs;
  fNtotal=0;
  fNSpill=0;
  int nrEvents=100;   
  unsigned int chval;
  fNAPVs = 4;
  int rflag = 0;

  // start timer
  TStopwatch* fTime =  new TStopwatch();
  fTime->Start();

  ControlSettingTab* fControl = ((ModuleTestApp*) ptr)->getControlSettingTab();
  int process_type = fControl->getProcessType();
  Histogrammer* fHistogrammer = Histogrammer::Instance();

  // Get the number of Raw Events to take from the RAW Label
  int RawEventNumber = fControl->getEventsToBeProcessed();
  int cyrcles = RawEventNumber/100;

  // Set the Range of the Progress Bar 
  fControl->getProgressBar()->SetRange(0, cyrcles);
  // Get Analysis Parameters
  int eventsForInitialisation = fControl->getInitEvent();
  int eventsForUpdate         = fControl->getUpdateEvent();

  TT6EventReader* myEventReader = new TT6EventReader();
  TT6ApvFactory::instance()->setEventReader(myEventReader);

  // Get Raw Data File Name
  string rfile_name  = fControl->getRawDataFileName();
  if (rfile_name.size()>0) {
    myEventReader->openRawFile(rfile_name.c_str());
    rflag = 1;
  }
  // Fill Histogram with General Informations
  TT6Timer theTT6Timer;
  cout << " Taking Data On : " << theTT6Timer.getDay()
       << "/" << theTT6Timer.getMonth() 
       << "/" << theTT6Timer.getYear() 
       << " At " << theTT6Timer.getHour() 
       << ":" << theTT6Timer.getMinute() 
       << ":" << theTT6Timer.getSecond() << endl;
  fHistogrammer->fillGeneralInfoHistogram(theTT6Timer.getYear(),
                                          theTT6Timer.getMonth(),
                                          theTT6Timer.getDay(), 
                                          theTT6Timer.getHour(), 
                                          theTT6Timer.getMinute(), 
                                          theTT6Timer.getSecond(), 
                                          4, 20);
  int ii = 0;
  while (ii < cyrcles &&  fControl->getState() == 1) {
    //
    // CLEAR THE FACTORY
    //
    myEventReader->nextEvent();

    for(int iev =0; iev < nrEvents; iev++) {
      for (int ich=0; ich<fNAPVs; ich++) {
        //
        // Here I am inside the APV loop....
        //
        // I need to create stuff for each of them, using a factory
        // I USE THE NUMBER AS THE INDEX, NOT THE POINTER!!!
        if (ich==0) fNtotal++;
        ApvAnalysis::RawSignalType rawInt; 
        for (int j = 0; j < 140; j++) {
	  chval = 300 + gRandom->Integer(25);
	  rawInt.push_back(chval);
        }
        fHistogrammer->fillRawHistograms(ich, rawInt);
        if (process_type == 2) {
	  myEventReader->setEvent(ich,ApvAnalysis::RawSignalType(rawInt.begin()+12,rawInt.end()),rflag);
          ApvAnalysis* myApv = TT6ApvFactory::instance()->getApv(ich);
	  myApv->newEvent();
	  myApv->updateCalibration();
	  if (myApv->pedestalCalculator().status()->isUpdating()) {
            if ((fNtotal - eventsForInitialisation)%eventsForUpdate == 1) {
	      //
	      // After Each Iteration.......
	      // get Pedestal and Raw Noise Values from Pedestal Calculator
	      //
	      ApvAnalysis::SubtractedSignalType stripPedestal = 
	                   myApv->pedestalCalculator().pedestal(); 
	      ApvAnalysis::SubtractedSignalType stripRNoise = 
	                   myApv->pedestalCalculator().rawNoise();
	      if ((fNtotal - eventsForInitialisation) == 1) {
	        TkApvMask::MaskType stripMask = myApv->mask().mask();
	        fHistogrammer->fillPedHistograms(ich, 1,
	                       stripPedestal, stripRNoise, stripMask);
	      } else {
		ApvAnalysis::SubtractedSignalType stripNoise = 
		            myApv->noiseCalculator().stripNoise();
		//		TT6NoiseCalculator* theTT6NoiseCalculator =
		//		     dynamic_cast<TT6NoiseCalculator*>(&(myApv->noiseCalculator()));
      		ApvAnalysis::SubtractedSignalType subtractedSignal;
		//		if (theTT6NoiseCalculator) {
		//		  subtractedSignal =
		//		     theTT6NoiseCalculator->stripCMPSubtractedSignal();
		//		} else {
		//		  subtractedSignal.resize(stripNoise.size(), 0.0);
		//		}
                TkApvMask::MaskType stripMask;
                myApv->mask().calculateMask(stripRNoise);
                stripMask = myApv->mask().mask();
                fHistogrammer->fillPedHistograms(ich, 2,
			       stripPedestal, stripRNoise, stripMask);
		myApv->mask().calculateMask(stripNoise);
		               stripMask = myApv->mask().mask();
		fHistogrammer->fillPedHistograms(ich, 3,
		               subtractedSignal, stripNoise, stripMask);     
              }
	    }
            //
            // Get Common Mode Values and Fill histograms    
            //  
            vector<float> CMValue = 
              myApv->commonModeCalculator().commonMode()->returnAsVector();
            fHistogrammer->fillCMHistograms(ich, CMValue);
	  }
        }
      }      
    }
    if (0) cout << " Spill Number " << ii << " Total Number of Events = " << fNtotal << endl;
    ii++;
    // Increment the Progress Bar
    fControl->getProgressBar()->Increment(1);
  }
  cout << " RawEventNumber = " << RawEventNumber << endl;
  if (fNtotal == RawEventNumber || fControl->getState() == 2 ) {
    fTime->Stop();
    cout << " DAQ Finished after : CPU Time " << fTime->CpuTime() 
         << " : Real Time  " << fTime->RealTime() << " sec  " << endl;
    delete fTime;
    //
    // I also kill all the APVs...
    //
    cout <<" CLEARING ALL THE CALIBRATIONS! (deleting APVs)"<<endl;
    TT6ApvFactory::instance()->clearAllCalibrations();
    if (rflag == 1) myEventReader->closeRawFile();
    delete myEventReader;  
    fControl->stopDataTaking();
    pthread_exit((void *)"Thanks for not smoking"); // casting to (void *)
  }
}
