//
//  The tab consists HV setting
//
#include <iostream.h>
#include <strstream.h>

#include "TROOT.h"
#include "TSystem.h"
#include "TGMsgBox.h"

#include "HVSettingTab.h"
 
ClassImp(HVSettingTab)

const float vValMax = 100.0;
const float iValMax = 10.0;
//
// Constructor  
//
HVSettingTab::HVSettingTab(const TGMainFrame *parent, const TGWindow* main, 
              UInt_t w, UInt_t h, UInt_t options, ULong_t back) 
            : TGCompositeFrame(main, w, h, options, back)
{

  fHVChannel = 0;
  fHVChannelMap.clear();   

  fHVCrateType = -1;

  fHVCrateNum   = 1;
  fHVBoardNum   = 0;
  fHVChannelNum = 0;

  fOn = false;
  CreateDisplay();

  fMonitorTimer = new TTimer(100);
  fMonitorTimer->Connect("Timeout()","HVSettingTab",this,"monitorHVStatus()");
}
// 
// Destructor 
//
HVSettingTab::~HVSettingTab() 
{
   delete     fHVCrateTypeBox;
   delete     fHVCrateNumNE;
   delete     fHVBoardNumNE;
   delete     fHVChannelNumNE;
   delete     fHVCrateTypeLab;
   delete     fHVCrateNumLab;
   delete     fHVBoardNumLab;
   delete     fHVChannelNumLab;
   delete     fHVRequestBut;
   delete     fHVReleaseBut;
   delete     fHVDevGFrame;


   delete     fVoltageNE;
   delete     fCurrentNE;
   delete     fRampUpNE;
   delete     fRampDownNE;
   delete     fVMonitorBar;
   delete     fIMonitorBar;
   delete     fVoltageLab;
   delete     fCurrentLab;
   delete     fRampUpLab;
   delete     fRampDownLab;
   delete     fVMonitorLab;
   delete     fIMonitorLab;
   delete     fSettingGFrame;

   delete     fHVReadBut;
   delete     fHVSetBut;
   delete     fHVReadAllBut;
   delete     fHVToggleBut; 
   delete     fActionGFrame; 
   delete     fReleaseAllBut;

   delete     fHVMessageFrame;
   delete     fHVMessageView;
   
   delete     fL;
   delete     fLLab;
   delete     fLBut;

   delete     fMonitorTimer;

   // Delete ALL Channels from the HV Channel Map   
   releaseAllHVChannels(0);
}
//
//
//
void HVSettingTab::CreateDisplay() 
{
  fL      = new TGLayoutHints(kLHintsTop|kLHintsLeft, 5, 5, 10, 10);
  fLLab   = new TGLayoutHints(kLHintsTop|kLHintsRight, 5, 1, 15, 15);
  fLBut   = new TGLayoutHints(kLHintsLeft ,10, 10, 10, 10);
  //
  // -- Device Settings 
  //
  fHVDevGFrame = new TGGroupFrame(this, " Device ", kHorizontalFrame);
  AddFrame(fHVDevGFrame);

  // Crate Type (SY127/SY527)
  fHVCrateTypeLab = new TGLabel(fHVDevGFrame, "Type");
  fHVDevGFrame->AddFrame(fHVCrateTypeLab, fL);

  fHVCrateTypeBox  = new TGComboBox(fHVDevGFrame, CM_CRATE_TYPE);
  fHVCrateTypeBox->Associate(this);
  fHVCrateTypeBox->AddEntry("SY127",1);
  fHVCrateTypeBox->AddEntry("SY527",2);
  fHVCrateTypeBox->Resize(65,25);
  fHVCrateTypeBox->Select(2);
  fHVDevGFrame->AddFrame(fHVCrateTypeBox, fL);

  // Crate Number
  fHVCrateNumLab    = new TGLabel(fHVDevGFrame, "Crate #");
  fHVDevGFrame->AddFrame(fHVCrateNumLab, fL);
  fHVCrateNumNE  = new TGNumberEntry(fHVDevGFrame, fHVCrateNum, 3, 
   NE_HV_DEVICE, TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative);
  fHVCrateNumNE->Associate(this);
  fHVDevGFrame->AddFrame(fHVCrateNumNE, fL);

  // Board Number 
  fHVBoardNumLab    = new TGLabel(fHVDevGFrame, "Board #");
  fHVDevGFrame->AddFrame(fHVBoardNumLab, fL);

  fHVBoardNumNE  = new TGNumberEntry(fHVDevGFrame, fHVBoardNum, 3, NE_HV_BOARD,
        TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative,
        TGNumberFormat::kNELLimitMinMax,0,9);  
  fHVBoardNumNE->Associate(this);
  fHVDevGFrame->AddFrame(fHVBoardNumNE, fL);

  // Channel Number
  fHVChannelNumLab    = new TGLabel(fHVDevGFrame, "Channel #");
  fHVDevGFrame->AddFrame(fHVChannelNumLab, fL);

  fHVChannelNumNE  = new TGNumberEntry(fHVDevGFrame, fHVChannelNum, 3, 
    NE_HV_CHAN, TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative,
    TGNumberFormat::kNELLimitMinMax,0,40);
  fHVChannelNumNE->Associate(this);
  fHVDevGFrame->AddFrame(fHVChannelNumNE, fL);

  // Request 
  fHVRequestBut = new TGTextButton(fHVDevGFrame, " &Request ", TB_HV_REQ);
  fHVRequestBut->Associate(this);
  fHVRequestBut->SetToolTipText("Request HV Channel");
  fHVDevGFrame->AddFrame(fHVRequestBut, fL);

  // Release 
  fHVReleaseBut = new TGTextButton(fHVDevGFrame, " &Release ", 
                                                                TB_HV_REL);
  fHVReleaseBut->Associate(this);
  fHVReleaseBut->SetToolTipText("Release HV Channel");
  fHVDevGFrame->AddFrame(fHVReleaseBut, fL);

  //
  // -- HV Settings and Monitoring
  //
  fSettingGFrame = new TGGroupFrame(this, " HV Setting ", kHorizontalFrame);
  fSettingGFrame->SetLayoutManager(new TGMatrixLayout(fSettingGFrame,0,6,5));
  AddFrame(fSettingGFrame);

  // V0  
  fVoltageLab = new TGLabel(fSettingGFrame, " V0 Set");
  fSettingGFrame->AddFrame(fVoltageLab,fL);

  fVoltageNE  = new TGNumberEntry(fSettingGFrame, 100.0, 5,
    NE_HV_VOLT, TGNumberFormat::kNESRealTwo, TGNumberFormat::kNEANonNegative);
  fVoltageNE->Associate(this);
  fSettingGFrame->AddFrame(fVoltageNE, fL);

  // Ramp Up 
  fRampUpLab = new TGLabel(fSettingGFrame, " Ramp Up Set");
  fSettingGFrame->AddFrame(fRampUpLab,fL);

  fRampUpNE  = new TGNumberEntry(fSettingGFrame, 1.0, 5,
    NE_HV_RUP, TGNumberFormat::kNESRealTwo, TGNumberFormat::kNEANonNegative);
  fRampUpNE->Associate(this);
  fSettingGFrame->AddFrame(fRampUpNE, fL);

  // Voltage Monitor
  fVMonitorLab = new TGLabel(fSettingGFrame, " Voltage Monitor ");
  fSettingGFrame->AddFrame(fVMonitorLab,fL);

  fVMonitorBar =  new TGHProgressBar(fSettingGFrame,TGProgressBar::kFancy,50);
  fVMonitorBar->SetBarColor("lightgreen");
  fVMonitorBar->ShowPosition(kTRUE, kFALSE, "%.0f V");
  fVMonitorBar->Reset();
  fVMonitorBar->SetRange(0, vValMax);
  fSettingGFrame->AddFrame(fVMonitorBar, fL);
  
  // I0  
  fCurrentLab = new TGLabel(fSettingGFrame, " I0 Set");
  fSettingGFrame->AddFrame(fCurrentLab,fLLab);

  fCurrentNE  = new TGNumberEntry(fSettingGFrame, 10.0, 5,
    NE_HV_CURR, TGNumberFormat::kNESRealTwo, TGNumberFormat::kNEANonNegative);
  fCurrentNE->Associate(this);
  fSettingGFrame->AddFrame(fCurrentNE, fL);

  // Ramp Up 
  fRampDownLab = new TGLabel(fSettingGFrame, " Ramp Down Set");
  fSettingGFrame->AddFrame(fRampDownLab,fL);

  fRampDownNE  = new TGNumberEntry(fSettingGFrame, 0.5, 5,
    NE_HV_RDOWN, TGNumberFormat::kNESRealTwo, TGNumberFormat::kNEANonNegative);
  fRampDownNE->Associate(this);
  fSettingGFrame->AddFrame(fRampDownNE, fL);

  // Current Monitor
  fIMonitorLab = new TGLabel(fSettingGFrame, " Current Monitor ");
  fSettingGFrame->AddFrame(fIMonitorLab,fL);

  fIMonitorBar =  new TGHProgressBar(fSettingGFrame,TGProgressBar::kFancy,50);
  fIMonitorBar->SetBarColor("lightblue");
  fIMonitorBar->ShowPosition(kTRUE, kFALSE, "%.0f");
  fIMonitorBar->Reset();
  fIMonitorBar->SetRange(0, iValMax);
  fSettingGFrame->AddFrame(fIMonitorBar, fL);
  //
  // -- Different Actions 
  //
  fActionGFrame = new TGGroupFrame(this, 
                     " HV Actions on Single/Multi Channels", kHorizontalFrame);
  AddFrame(fActionGFrame);

  // Read A Single Channel
  fHVReadBut = new TGTextButton(fActionGFrame, " &Read a Ch", TB_HV_SREAD);
  fHVReadBut->Associate(this);
  fHVReadBut->Resize(95,25);
  fHVReadBut->SetToolTipText("Read HV Channel");
  fActionGFrame->AddFrame(fHVReadBut, fLBut);

  // Set A Single Channel
  fHVSetBut = new TGTextButton(fActionGFrame, " &Set a Ch ", TB_HV_SET);
  fHVSetBut->Associate(this);
  fHVSetBut->Resize(95,25);
  fHVSetBut->SetToolTipText("Set HV Channel");
  fActionGFrame->AddFrame(fHVSetBut, fLBut);

  // Toggle 
  fHVToggleBut = new TGTextButton(fActionGFrame, " &Toggle a Ch", TB_HV_TOG);
  fHVToggleBut->Associate(this);
  fHVToggleBut->Resize(95,25);
  fHVToggleBut->SetToolTipText("Toggle HV Channel");
  fActionGFrame->AddFrame(fHVToggleBut, fLBut);

  // Read All Requested Channels
  fHVReadAllBut = new TGTextButton(fActionGFrame, 
                        " &Read All ", TB_HV_AREAD);
  fHVReadAllBut->Associate(this);
  fHVReadAllBut->Resize(130,25);
  fHVReadAllBut->SetToolTipText("Read All Requested HV Channels");
  fActionGFrame->AddFrame(fHVReadAllBut, fLBut);

  // Release All Channels 
  fReleaseAllBut = new TGTextButton(fActionGFrame, " &Release All ", TB_HV_AREL);
  fReleaseAllBut->Associate(this);
  fReleaseAllBut->Resize(150,25);
  fReleaseAllBut->SetToolTipText("Release All Requested HV Channels");
  fActionGFrame->AddFrame(fReleaseAllBut, fLBut);
  //
  // Text Field for Messages 
  //
  fHVMessageFrame = new TGHorizontalFrame(this, 550, 150); 
  AddFrame(fHVMessageFrame);

  fHVMessageView = new TGTextView(fHVMessageFrame, 550, 250);
  fHVMessageFrame->AddFrame(fHVMessageView, fL);
}
// 
// Handle user actions 
//
Bool_t HVSettingTab::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2) 
{
    // Handle messages
  switch (GET_MSG(msg)) {

    case kC_COMMAND:
      switch (GET_SUBMSG(msg)) {

        case kCM_BUTTON:
          switch (parm1) {

	  case TB_HV_REQ:
            {
              if (fHVCrateType == -1) {
                int retval;
                new TGMsgBox(fClient->GetRoot(), this, " HV  Message",
		 "Please Select Device Type First and Try Again !!",
                 kMBIconExclamation, kMBOk, &retval); 

              } else if (fHVCrateType == 1) {
                requestSY127Channel();
	      } else if (fHVCrateType == 2) {
                requestChannel();
              }
              break; 
            }

	  case TB_HV_REL:
            {
              releaseSingleHVChannel();
              break; 
            }

	  case TB_HV_SREAD:
            {
              readSingleHVSetting();
              break; 
            }

	  case TB_HV_SET:
            {
              applyHVSetting();
              break; 
            }

	  case TB_HV_TOG:
            {
              toggleHVSetting();
              fMonitorTimer->Start();
              break; 
            }

	  case TB_HV_AREAD:
            {
              readAllHVSettings();
              break; 
            }

	  case TB_HV_AREL:
            {
              releaseAllHVChannels(1);
              break; 
            }

            default:
              break;
          }
        case kCM_COMBOBOX:

          switch (parm1) {
            case CM_CRATE_TYPE:
              switch (parm2) {
	        case 1:
		  {
                    releaseAllHVChannels(0);
		    fHVCrateType = 1;
                    fHVBoardNumNE->SetState(kFALSE);  
                    break;
                  }

                case 2:
                  {
                    releaseAllHVChannels(0);
		    fHVCrateType = 2;
                    fHVBoardNumNE->SetState(kTRUE);
                    break;
                  }
                    
                default:
		  {
                    fHVCrateType = -1;
                    break;
                  }
	      }
          }
      }
   }
   return kTRUE;
}
// 
// -- Read Channel Identity
// 
int HVSettingTab::readChannelIdentity(){
  if (fHVCrateType == -1) {
    int retval;
    new TGMsgBox(fClient->GetRoot(), this, " HV  Message",
		 "Please Select Device Type First and Try Again !!",
                 kMBIconExclamation, kMBOk, &retval); 
    return -1; 
  } 
  fHVCrateNum   = fHVCrateNumNE->GetIntNumber();
  fHVChannelNum = fHVChannelNumNE->GetIntNumber();
  int IdChannel;

  if (fHVCrateType == 2) {
    fHVBoardNum   = fHVBoardNumNE->GetIntNumber();
    IdChannel = 20*fHVBoardNum + fHVChannelNum;
  } else if (fHVCrateType == 1) {
    fHVBoardNum   = int(fHVChannelNum/4);   
    IdChannel = fHVChannelNum;    
  }
  return IdChannel;
}
// 
// -- Request Channel 
// 
void HVSettingTab::requestChannel(){
  int IdChannel = this->readChannelIdentity();
  if (IdChannel == -1) return;
  ostrstream strval;
  const char *cstr;  
  strval << " Requested HV Channel with Board #" << fHVBoardNum 
         << " Channel #" << fHVChannelNum << ends;
  cstr = strval.str();
  fHVMessageView->AddLine(cstr);
  delete [] cstr;         
}
// 
// -- Request Channel 
// 
void HVSettingTab::requestSY127Channel(){
  int IdChannel = this->readChannelIdentity();
  if (IdChannel == -1) return;
  ostrstream strval;
  const char *cstr;  
  strval << " Requested HV Channel with Board #" << fHVBoardNum
         << " Channel #" << fHVChannelNum << ends;
  cstr = strval.str();
  fHVMessageView->AddLine(cstr);
  delete [] cstr;         
}
// 
// -- Release A Single HV Channel 
// 
void HVSettingTab::releaseSingleHVChannel(){
  
  int IdChannel = this->readChannelIdentity();
  if (IdChannel == -1) return;
  ostrstream strval;
  const char *cstr;  
  strval << " Releasing  HV Channel with Board #" << fHVBoardNum
        << " Channel #" << fHVChannelNum << ends;
  cstr = strval.str();
  fHVMessageView->AddLine(cstr);
  delete [] cstr; 
}
// 
// -- Read HV Setting 
// 
void HVSettingTab::readSingleHVSetting(){
  int IdChannel = this->readChannelIdentity();
  if (IdChannel == -1) return;
  ostrstream strhv;
  strhv << " Reading from Board #"<< fHVBoardNum
        << " Channel #" << fHVChannelNum  << ends;
  const char *cstrhv = strhv.str();      
  fHVMessageView->AddLine(cstrhv);
  delete [] cstrhv;
}
// 
// -- Read  HV Setting of All Requested Channels
// 
void HVSettingTab::readAllHVSettings() {
  cout << " Not Implimented Yet !!!!!! " << endl;
}
// 
// -- Apply HV Setting 
// 
void HVSettingTab::applyHVSetting(){
  cout << " Not Implimented Yet !!!!!! " << endl;
}
// 
// -- Toggle HV Setting 
// 
void HVSettingTab::toggleHVSetting(){
  int IdChannel = this->readChannelIdentity();
  if (IdChannel == -1) return;
  ostrstream strval;
  strval << " Change Status for Board #" << fHVBoardNum 
         << " Channel #" << fHVChannelNum  << ends ;
  const char *cstr = strval.str();
  fHVMessageView->AddLine(cstr);
  delete [] cstr;

  if (!fOn) {
    fOn = true;
  } else {
    fOn = false;
  }
}
// 
// -- Release All Requested HV Channels and clear the MAP
// 
void HVSettingTab::releaseAllHVChannels(int iflag) {
  ostrstream strval;
  strval << " Releasing All HV Channels " << ends;
  const char *cstr = strval.str();
  fHVMessageView->AddLine(cstr);
  delete [] cstr;
}
//
// -- Read and Load Configuration
//
void HVSettingTab::readAndLoadConfig(string fname_config) {
  cout << " Not Yet Implemented " << endl;
}
//
// Monitor HV State Using TTimer
//
void HVSettingTab::monitorHVStatus() {
  float fVMonitorVal = fVMonitorBar->GetPosition();
  float fIMonitorVal = fIMonitorBar->GetPosition();
  if (fOn && fVMonitorVal < (vValMax - 1.0) ) {
    fVMonitorVal += 1.0;
    fIMonitorVal += 0.1;
    fVMonitorBar->SetPosition(fVMonitorVal);
    fIMonitorBar->SetPosition(fIMonitorVal);
  } else if (!fOn) {
    fVMonitorVal -= 1.0;
    fIMonitorVal -= 0.1;
    fVMonitorBar->SetPosition(fVMonitorVal);
    fIMonitorBar->SetPosition(fIMonitorVal);    
  } else {
    fMonitorTimer->Stop();
  }
}
//
// -- Actions for Automatic Running
//
void HVSettingTab::doItAutoRun(string fname_config) {
  cout << " Not Yet Implemented " << endl;
}
