//
// -- The tab consists Histogram Viewing  Settings
//
#include <iostream.h>
#include <strstream.h>
#include <unistd.h>
#include <stdlib.h>
#include <string>
#include <fstream>

#include "TROOT.h"
#include "TSystem.h"
#include "TKey.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TGMsgBox.h"
#include "TGFileDialog.h"
#include "TBrowser.h"
#include "TH1.h"
#include "PrintSetup.h"
#include "HistogramViewingTab.h" 
#include "TextViewer.h"
#include "ConfigBadChannel.h"

static const string sHistoType[]={"Pedestal Vs Strips",
                                  "Pedestal Distribution",
                                  "Raw Noise Vs Strips",
                                  "Raw Noise Distribution",
                                  "Noise Vs Strips",
                                  "Noise Distribution",
                                  "Common Mode Distribution", 
			          "Calibration Charge ",
		                  "Maximum Charge Vs Strips",
                                  "Maximum Charge Distribution",
			          "Rise Time Vs Strips",
                                  "Rise Time Distribution",
                                  "Laser Pulse vs Strips"};
                                 
static const string sMacroFileName[]={"plot_pedestal_vs_strip.C",
                                      "plot_pedestal.C",
                                      "plot_rawnoise_vs_strip.C",
                                      "plot_rawnoise.C",
                                      "plot_noise_vs_strip.C",
                                      "plot_noise.C",
                                      "plot_cm.C",
                                      "plot_calibration_charge.C",
                                      "plot_max_charge_vs_strip.C",
                                      "plot_max_charge.C",
                                      "plot_tau_vs_strip.C",
                                      "plot_tau.C",
                                      "plot_laser_pulse_vs_strip.C"};

static const string sTT6MacroFileName[]={"plot_pedestal_vs_strip_tt6.C",
                                      "plot_pedestal_tt6.C",
                                      "plot_rawnoise_vs_strip_tt6.C",
                                      "plot_rawnoise_tt6.C",
                                      "plot_noise_vs_strip_tt6.C",
                                      "plot_noise_tt6.C",
                                      "plot_cm_tt6.C",
                                      "plot_calibration_charge_tt6.C",
                                      "plot_max_charge_vs_strip_tt6.C",
                                      "plot_max_charge_tt6.C",
                                      "plot_tau_vs_strip_tt6.C",
                                      "plot_tau_tt6.C",
                                      "plot_laser_pulse_vs_strip_tt6.C"};
//
// Constructor  
//
HistogramViewingTab::HistogramViewingTab(const TGMainFrame *parent, const TGWindow* main, 
              UInt_t w, UInt_t h, UInt_t options, ULong_t back) 
            : TGCompositeFrame(main, w, h, options, back)
{
  CreateDisplay();
  fHistogramType = -1;
  nAPVsToBePlotted  = -1;
  IdAPV     = -1;
  moduleFlag = -1;
}
// 
// Destructor 
//
HistogramViewingTab::~HistogramViewingTab() 
{
  
   delete    fHistoCanvas;
   delete    fHistoFrame;

   delete    fFileBut;  
   delete    fBrowserBut;
   delete    fClearBut;  
   delete    fMacroBut; 
   delete    fSaveBut;  
   delete    fPrintBut;
   delete    fAnalBut;   
   delete    fLyonSelectCB;   
   delete    fButtonFrame;

   // Delete  Plot Type widgets
   
   for (int i = 0; i < 3; i++) {
     delete  fPlotTypeRB[i];
     delete  fPlotTypeEntry[i];
     delete  fPlotTypeFrame[i];
   }
   delete    fHistoTypeLab;
   delete    fHistoTypeBox;
   delete    fPlotBut;  
   delete    fPlotTypeGFrame;
   delete    fTypeFrame;

   delete    fL;
   delete    fLType;
   delete    fLBut; 
   delete    fLAnalBut; 
   
   if (fRootFile) delete fRootFile;
}
void HistogramViewingTab::CreateDisplay() 
{
  fL = new TGLayoutHints(kLHintsTop, 5, 5, 5, 5);
  fLType = new TGLayoutHints(kLHintsNormal, 3, 3, 2, 2);
  fLBut = new TGLayoutHints(kLHintsTop| kLHintsExpandX,5, 5, 10, 10);
  fLAnalBut = new TGLayoutHints(kLHintsTop | kLHintsLeft| kLHintsExpandX,
                                        5, 5, 30, 30);
  // Histogram Frame 

  fHistoFrame = new TGHorizontalFrame(this, 10, 10);
  AddFrame(fHistoFrame);

  // Histogram Canvas 
  fHistoCanvas = new TRootEmbeddedCanvas("Histogram Canvas", this, 450, 430);
  fHistoFrame->AddFrame(fHistoCanvas, fL);

  // Buttons 
  fButtonFrame = new TGVerticalFrame(fHistoFrame, 10, 10);
  fHistoFrame->AddFrame(fButtonFrame, fL); 
  // File 
  fFileBut = new TGTextButton(fButtonFrame, "&Load File", TB_FILE);
  fFileBut->Resize(100,30);
  fFileBut->Associate(this);
  fFileBut->SetToolTipText("Load Root File");
  fButtonFrame->AddFrame(fFileBut,fLBut);
  // Browser 
  fBrowserBut = new TGTextButton(fButtonFrame, "&Browser", TB_BROW);
  fBrowserBut->Associate(this);
  fBrowserBut->SetToolTipText("Open Browser");
  fButtonFrame->AddFrame(fBrowserBut,fLBut);
  // Clear 
  fClearBut = new TGTextButton(fButtonFrame, "&Clear Canvas", TB_CLEAR);
  fClearBut->Associate(this);
  fClearBut->SetToolTipText("Clear Histogram Canvas");
  fButtonFrame->AddFrame(fClearBut,fLBut);
  // Macro 
  fMacroBut = new TGTextButton(fButtonFrame, "&Exectute Macro", TB_MACRO);
  fMacroBut->Associate(this);
  fMacroBut->SetToolTipText("Execute Macro");
  fButtonFrame->AddFrame(fMacroBut,fLBut);
  // Save 
  fSaveBut = new TGTextButton(fButtonFrame, "&Save Canvas", TB_SAVE);
  fSaveBut->Associate(this);
  fSaveBut->SetToolTipText("Save the Canvas in eps file");
  fButtonFrame->AddFrame(fSaveBut,fLBut);
  // Print 
  fPrintBut = new TGTextButton(fButtonFrame, "&Print Canvas", TB_PRINT);
  fPrintBut->Associate(this);
  fPrintBut->SetToolTipText("Print the Canvas");
  fButtonFrame->AddFrame(fPrintBut,fLBut);
  // Selection for HibridDialog root File 
  fLyonSelectCB = new TGCheckButton(fButtonFrame, "Type 2 File", CB_LYON_SEL);
  fLyonSelectCB->Associate(this);
  fButtonFrame->AddFrame(fLyonSelectCB, fLBut);
  // Analysis of Bad Channel 
  fAnalBut = new TGTextButton(fButtonFrame, "&Analyse Bad Chan", TB_ANAL);
  fAnalBut->Associate(this);
  fAnalBut->SetToolTipText("Analyse Bad Channel");
  fButtonFrame->AddFrame(fAnalBut,fLAnalBut);

  // Histograms and  Plotting Types 
  fTypeFrame = new TGHorizontalFrame(this, 10, 10);
  AddFrame(fTypeFrame, fL);
    
  fPlotTypeGFrame = new TGGroupFrame(fTypeFrame, "Plot Types", kVerticalFrame);
  fTypeFrame->AddFrame(fPlotTypeGFrame);

  // Plot Type (one APV/ more than one APV/Module)  

  for (int i = 0; i < 3; i++) {
    fPlotTypeFrame[i] = new TGHorizontalFrame(fPlotTypeGFrame, 30,20);
    fPlotTypeGFrame->AddFrame(fPlotTypeFrame[i],fLType);
    if (i == 0) {
      fPlotTypeRB[i] = new TGRadioButton(fPlotTypeFrame[i],
                        " One APV,    ID ",RB_HISTO_PLOT_A);
    } else if (i == 1) {
        fPlotTypeRB[i] = new TGRadioButton(fPlotTypeFrame[i],
                        " Multi APVs,  #  ",RB_HISTO_PLOT_B);
    } else {
        fPlotTypeRB[i] = new TGRadioButton(fPlotTypeFrame[i],
                          " Module, #APVs",  RB_HISTO_PLOT_C); 
    }
    fPlotTypeRB[i]->Associate(this);
    fPlotTypeFrame[i]->AddFrame(fPlotTypeRB[i], fLType);
  
    fPlotTypeEntry[i] = new TGTextEntry(fPlotTypeFrame[i], "");
    fPlotTypeEntry[i]->Resize(30,17);
    fPlotTypeFrame[i]->AddFrame(fPlotTypeEntry[i], fLType);
  }
  /*
  fPlotTypeFrame[0] = new TGTextEntry(fPlotTypeFrame, "");
  fPlotTypeFrame[1] = new TGTextEntry(fPlotTypeFrame, "");
  fPlotTypeFrame[2] = new TGTextEntry(fPlotTypeFrame, "");


  fPlotTypeRB[0] = new TGRadioButton(fPlotTypeFrame,
                        " One APV, ID =",RB_HISTO_PLOT_A);
  fPlotTypeRB[1] = new TGRadioButton(fPlotTypeFrame,
                      " Multi APV, # = ",RB_HISTO_PLOT_B);
  fPlotTypeRB[2] = new TGRadioButton(fPlotTypeFrame,
  " Module, #APVs = ",  RB_HISTO_PLOT_C); 


  fPlotTypeEntry[1] = new TGTextEntry(fPlotTypeFrame, "");
  fPlotTypeEntry[2] = new TGTextEntry(fPlotTypeFrame, "");
  */

  // Histogram Selection 
  fHistoTypeLab = new TGLabel(fTypeFrame, " Histogram Type");
  fTypeFrame->AddFrame(fHistoTypeLab, fL);
  fHistoTypeBox = new TGComboBox(fTypeFrame, CM_HISTO_TYPE);
  fHistoTypeBox->Associate(this);
  for (int i = 0; i < 13; i++) {
    fHistoTypeBox->AddEntry(sHistoType[i].c_str(),i);
  }
  fHistoTypeBox->Resize(200,25);
  fHistoTypeBox->Select(0);
  fTypeFrame->AddFrame(fHistoTypeBox, fL);

  // Plot 
  fPlotBut = new TGTextButton(fTypeFrame, "&Plot Histogram", TB_PLOT);
  fPlotBut->Associate(this);
  fPlotBut->SetToolTipText("Plot Selected Histogram(s) with Preferred Type");
  fTypeFrame->AddFrame(fPlotBut,fL);
}
// 
// Handle user actions 
//
Bool_t HistogramViewingTab::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2) 
{
    // Handle messages
  switch (GET_MSG(msg)) {

    case kC_COMMAND:
      switch (GET_SUBMSG(msg)) {

        case kCM_RADIOBUTTON:
          switch (parm1) {

            case RB_HISTO_PLOT_A:
              fPlotTypeEntry[0]->SetText(" ");
              fPlotTypeEntry[0]->SetState(kTRUE);
              fPlotTypeRB[1]->SetState(kButtonUp);
              fPlotTypeEntry[1]->SetText(" ");
              fPlotTypeEntry[1]->SetState(kFALSE);
              fPlotTypeRB[2]->SetState(kButtonUp);
              fPlotTypeEntry[2]->SetText(" ");
              fPlotTypeEntry[2]->SetState(kFALSE);
              break;

            case RB_HISTO_PLOT_B:
              fPlotTypeRB[0]->SetState(kButtonUp);
              fPlotTypeEntry[0]->SetText(" ");
	      fPlotTypeEntry[0]->SetState(kFALSE);
              fPlotTypeEntry[1]->SetText(" ");
	      fPlotTypeEntry[1]->SetState(kTRUE);
              fPlotTypeRB[2]->SetState(kButtonUp);
              fPlotTypeEntry[2]->SetText(" ");
	      fPlotTypeEntry[2]->SetState(kFALSE);
              break;

            case RB_HISTO_PLOT_C:
              fPlotTypeRB[0]->SetState(kButtonUp);
              fPlotTypeEntry[0]->SetText(" ");
	      fPlotTypeEntry[0]->SetState(kFALSE);
              fPlotTypeRB[1]->SetState(kButtonUp);
              fPlotTypeEntry[1]->SetText(" ");
	      fPlotTypeEntry[1]->SetState(kFALSE);
              fPlotTypeEntry[2]->SetText(" ");
	      fPlotTypeEntry[2]->SetState(kTRUE);
              break;
          }
        case kCM_BUTTON:
          switch (parm1) {
        
            case TB_FILE:
              loadFile();
              break;
 
            case TB_BROW:
              {
	        TBrowser* b = new TBrowser();
                b->Show();
                break;
              }

            case TB_CLEAR:
              clearCanvas();
              break;

            case TB_MACRO:
              executeMacro();
              break;

            case TB_SAVE:
              saveCanvas();
              break;

            case TB_PRINT:

              printCanvas();
              break;

            case TB_PLOT:
              if (fLyonSelectCB->GetState() == kButtonDown) {
                fLyonFlag = 1;
              } else {
                fLyonFlag = 0;
              }
              plotHistogram();
              break;

            case TB_ANAL:
              analyseBadChannels();
              break;

            default:
              break;
          }

        case kCM_COMBOBOX:

           switch (parm1) {
             case CM_HISTO_TYPE:
               switch (parm2) {
                 case 0:
                   fHistogramType = 0;
                 case 1:
                   fHistogramType = 1;
                 case 2:
                   fHistogramType = 2;
                 case 3:
                   fHistogramType = 3;
                 case 4:
                   fHistogramType = 4;
                 case 5:
                   fHistogramType = 5;
                 case 6:
                   fHistogramType = 6;
                 case 7:
                   fHistogramType = 7;
                 case 8:
                   fHistogramType = 8;
                 case 9:
                   fHistogramType = 9;
                 case 10:
                   fHistogramType = 10;
                 case 11:
                   fHistogramType = 11;
                 case 12:
                   fHistogramType = 12;
	       }
           }
      }
   }
   return kTRUE;
}
// 
// Load Root File
//
void HistogramViewingTab::loadFile()
{
  char *fRootFileName; 
  TGFileInfo fi;
  const char *fHistoFileType[] = {"Root Files",     "*.root",
				  "Hbook Files",    "*.hbook"};
  
  fi.fFileTypes = fHistoFileType;
  new TGFileDialog(fClient->GetRoot(), this, kFDOpen, &fi);

  if (!fi.fFilename) {
    int retval;
    new TGMsgBox(fClient->GetRoot(), this, "Analysis Message",
	" File Does not exist OR not a ROOT file !! ",
         kMBIconExclamation, kMBOk, &retval);  
    return;
  } else {
     if (fRootFile) delete fRootFile;
     fRootFileName = fi.fFilename;
     fRootFile = new TFile (fRootFileName);
     cout << " File :: "<< fRootFileName << " is loaded "  << endl;
  }     
}
// 
// Plot Histogram(s)
//
void HistogramViewingTab::plotHistogram()
{
  if (!fRootFile) {
    int retval;
    new TGMsgBox(fClient->GetRoot(), this, "Analysis Message",
		 "Please Load the Root File First and Try Again !!",
                 kMBIconExclamation, kMBOk, &retval); 
    return; 
  } else {
    // Get Plot Type and Number of APVs to be plotted from Widget 
    if (fPlotTypeRB[0]->GetState() == kButtonDown) {
      nAPVsToBePlotted = 1;
      IdAPV            = atoi(fPlotTypeEntry[0]->GetText());
      moduleFlag       = -1;
    } else if (fPlotTypeRB[1]->GetState() == kButtonDown) {
      nAPVsToBePlotted = atoi(fPlotTypeEntry[1]->GetText());
      if (nAPVsToBePlotted == 1) 
        IdAPV = 1;
      else 
        IdAPV         = -1;
      moduleFlag      = -1;
    } else if (fPlotTypeRB[2]->GetState() == kButtonDown) {
      moduleFlag = 1;
      nAPVsToBePlotted = atoi(fPlotTypeEntry[2]->GetText());
      IdAPV = -1;
    }
    // Get Histogram Type to be Plotted from the Combo Box
    fHistogramType = fHistoTypeBox->GetSelected();

    if (nAPVsToBePlotted == -1 || fHistogramType == -1) {
      int retval;
      new TGMsgBox(fClient->GetRoot(), this, "Analysis Message",
	  "Please Select Histogram Type and(or) Plot Type First then Try !!",
           kMBIconExclamation, kMBOk, &retval);  
    } else {
      if (nAPVsToBePlotted > 0 &&  nAPVsToBePlotted < 7){
        if (nAPVsToBePlotted == 1) {
          cout << " Plotting Histogram Type " << sHistoType[fHistogramType] 
	       << " For one APV :: ID = "  << IdAPV << endl;
        } else if (nAPVsToBePlotted > 1 && moduleFlag < 0) {
         cout << " Plotting Histogram Type " << sHistoType[fHistogramType] 
              << " For multi APVs :: # = " << nAPVsToBePlotted << endl;
        } else if (nAPVsToBePlotted > 1 && moduleFlag == 1) {
        cout << " Plotting Histogram Type " << sHistoType[fHistogramType] 
             << " For Module with APVs :: # = " << nAPVsToBePlotted << endl;
        }
        executeStandardMacro();
      }    
    }
  }
}
// 
// Clear Canvas
//
void HistogramViewingTab::clearCanvas()
{
  cout << " Clear Canvas " << endl;
  fHistoCanvas->GetCanvas()->Divide(1,1);
  fHistoCanvas->GetCanvas()->Clear();
  fHistoCanvas->GetCanvas()->Update();
}
// 
// Save Canvas
//
void HistogramViewingTab::saveCanvas()
{
  cout << " Save Canvas  " << endl;
  char *fSaveFileName; 
  TGFileInfo fo;
  new TGFileDialog(fClient->GetRoot(), this, kFDSave, &fo);

  if (!fo.fFilename) {
    int retval;
    new TGMsgBox(fClient->GetRoot(), this, "Analysis Message",
	" Error !! File Cannot be Opened  ",
         kMBIconExclamation, kMBOk, &retval);  
    return;
  } else {
     fSaveFileName = fo.fFilename;
     fHistoCanvas->GetCanvas()->SaveAs(fSaveFileName);
  }
}
// 
// Print Canvas
//
void HistogramViewingTab::printCanvas()
{
  Int_t ret, flag;
  char name[100];
  cout << " Print Canvas " << endl;
  new PrintSetup(fClient->GetRoot(), this, 600, 500, &ret, &flag, name);
  if(name != "laser07" || name != "laser01") return;

  switch (flag) {
    case 1:
      {
        fHistoCanvas->GetCanvas()->SaveAs("canvas_print.ps");
        string command = "lpr -P";
        command += name;
        command += " canvas_print.ps";
        cout << " Command to execute is " << command << endl;
        //     gSystem->Exec(command.c_str()); 
        command = "rm -f canvas_print.ps";
        gSystem->Exec(command.c_str());
        break;
      }

    case 2:
      fHistoCanvas->GetCanvas()->SaveAs(name);
      break;
  }
}
// 
// Execute Standard Macros
//
void HistogramViewingTab::executeStandardMacro()
{
  if (!fRootFile) {
    int retval;
    new TGMsgBox(fClient->GetRoot(), this, "Analysis Message",
		 "Please Load the Root File First and Try Again !!",
                 kMBIconExclamation, kMBOk, &retval);  
  } else {
    // Divide the Canvas According to the Histogram Plot Type 
  
    if (nAPVsToBePlotted == 1 || moduleFlag != -1) { 
      fHistoCanvas->GetCanvas()->Clear();
      fHistoCanvas->GetCanvas()->Divide(1,1);
    } else if (nAPVsToBePlotted == 2) {
      fHistoCanvas->GetCanvas()->Clear();
      fHistoCanvas->GetCanvas()->Divide(1,2);
    } else if (nAPVsToBePlotted == 3 ) {
      fHistoCanvas->GetCanvas()->Clear();
      fHistoCanvas->GetCanvas()->Divide(1,3);
    } else if (nAPVsToBePlotted == 4 ) {
      fHistoCanvas->GetCanvas()->Clear();
      fHistoCanvas->GetCanvas()->Divide(2,2);
    } else if (nAPVsToBePlotted == 5 || nAPVsToBePlotted == 6) { 
      fHistoCanvas->GetCanvas()->Clear();
      fHistoCanvas->GetCanvas()->Divide(2,3);
    } 
    ostrstream strval;
    if (fLyonFlag == 1) {
      strval << "macro/" << sMacroFileName[fHistogramType]
             << "(" << moduleFlag << ", " << nAPVsToBePlotted
             << ", " << IdAPV << ", " 
             << "\"Histogram Canvas\")" << ends;
    } else {
      strval << "macro/" << sTT6MacroFileName[fHistogramType]
             << "(" << moduleFlag << ", " << nAPVsToBePlotted
             << ", " << IdAPV << ", " 
             << "\"Histogram Canvas\")" << ends;
    } 
    const char *cstr = strval.str();
    cout << cstr << endl;
    Int_t error = 0;
    Int_t result = gROOT->Macro(cstr, &error);
    delete [] cstr;
  }
}
// 
// Execute User Macros
//
void HistogramViewingTab::executeMacro()
{
  if (!fRootFile) {
    int retval;
    new TGMsgBox(fClient->GetRoot(), this, "Analysis Message",
		 "Please Load the Root File First and Try Again !!",
                 kMBIconExclamation, kMBOk, &retval);
    return;  
  } else {
    char *fMacroFileName; 
    TGFileInfo fmacro;
    const char* macroTypes[] = {"Macro Files", "*.C"}; 
    fmacro.fFileTypes = macroTypes;
    new TGFileDialog(fClient->GetRoot(), this, kFDOpen, &fmacro);
    if (!fmacro.fFilename) {
      int retval;
      new TGMsgBox(fClient->GetRoot(), this, "Analysis Message",
		 "Macro File Cannot be Opened ",
                 kMBIconExclamation, kMBOk, &retval);
      return;  
    } else {
      fMacroFileName = fmacro.fFilename;
      Int_t error = 0;
      Int_t result = gROOT->Macro(fMacroFileName, &error);
    } 
  }
}
// 
// Get Canvas
//
TCanvas* HistogramViewingTab::getCanvas() {
  TCanvas* fTCanvas = fHistoCanvas->GetCanvas();
  return fTCanvas;
}
//
// -- Analyse Bad Channels
//
void HistogramViewingTab::analyseBadChannels(){
  ostrstream strval;
  ofstream   fBadChannelFile;

  int   nChip;
  int   cutType;
  float cut_low_val[4];
  float cut_high_val[4];

  if (!fRootFile) {
    cout << " Please Load the Root File First " << endl;  
  } else {
    string file_name = fRootFile->GetName();
    cout << " Analysing the Root File " << file_name << endl;  
    file_name.replace(file_name.end()-4,file_name.end(),"out");
    cout << " The Output File Name " << file_name << endl;
    fBadChannelFile.open(file_name.c_str(), ios::out);
    // Read Parameter Values from GUI 
    new ConfigBadChannel(this, 500, 300, &nChip, &cutType, cut_low_val, cut_high_val);
    strval << "macro/analyse_badchannel_tt6.C"
         << "(" << nChip << ", " 
         << cutType << ", "
         << cut_low_val[0] << ", " << cut_low_val[1] << ", " 
         << cut_low_val[2] << ", " << cut_low_val[3] << ", " 
         << cut_high_val[0] << ", " << cut_high_val[1] << ", " 
         << cut_high_val[2] << ", " << cut_high_val[3] << ", "
	   << "\"Histogram Canvas\")" << ends;
    const char *cstr = strval.str();
    cout << cstr << endl;
    Int_t error = 0;
    Int_t result = gROOT->Macro(cstr, &error);
    delete [] cstr;  

    TH1* fThFlag = (TH1 *) gROOT->FindObject("bad_strip_flag_vs_str");
    if (fThFlag) {
      int nBin = fThFlag->GetNbinsX();
      fBadChannelFile << "  List of Bad Channels " << endl;
      fBadChannelFile << " ====================== " << endl; 
      for (int k = 1; k < nBin+1; k++) {
        int fval =  static_cast<int>(fThFlag->GetBinContent(k));
        if (fval != 0) {
          fBadChannelFile << "Strip Number " << k << " Flag = " << fval << endl;
        }
      }
    }
    fBadChannelFile.close();    
    TextViewer *tv = new TextViewer(this, 250, 500);
    tv->LoadFile(file_name.c_str());
    tv->Popup();
  }
}
