#include "TextViewer.h"

/* 
 * Create a simple viewer. This widget contains a readonly Text
 * area to display text content and an 'OK' button at the bottom
 * which terminates the viewer when pressed.
 */
TextViewer::TextViewer(const TGWindow *main, UInt_t w, UInt_t h)
          : TGTransientFrame(gClient->GetRoot(), main, w, h)
{
     // The text area
   fView = new TGTextView(this, w, h, kSunkenFrame | kDoubleBorder);
   fL1   = new TGLayoutHints(kLHintsExpandX | kLHintsExpandY, 3, 3, 3, 3);
   AddFrame(fView, fL1);

     // The Ok button
   fOK = new TGTextButton(this, "  &OK  ");
   fL2 = new TGLayoutHints(kLHintsBottom | kLHintsCenterX, 0, 0, 5, 5);
   AddFrame(fOK, fL2);

   SetTitle();
   MapSubwindows();
   Resize(GetDefaultSize());

     // position relative to the parent's window
   Window_t wdum;
   int ax, ay;
   gVirtualX->TranslateCoordinates(main->GetId(), GetParent()->GetId(),
                          ((TGFrame *) main)->GetWidth() - (fWidth >> 1),
                          (((TGFrame *) main)->GetHeight() - fHeight) >> 1,
                          ax, ay, wdum);
   Move(ax, ay);
   SetWMPosition(ax, ay);
}

/* 
 *  Destroy viewer dialog
 */
TextViewer::~TextViewer()
{
   delete fView;
   delete fOK;
   delete fL1;
   delete fL2;
}
/*
 * Set title in viewer window
 */
void TextViewer::SetTitle()
{
   TGText *txt = GetViewer()->GetText();
   Bool_t untitled = !strlen(txt->GetFileName()) ? kTRUE : kFALSE;

   char title[256];
   if (untitled)
      sprintf(title, "Text viewer - Untitled");
   else
      sprintf(title, "Text Viewer - %s", txt->GetFileName());

   SetWindowName(title);
   SetIconName(title);
}

/* 
 * Show viewer, e.g make the viewer visible 
 */
void TextViewer::Popup()
{
   MapWindow();
}

/* 
 * Load a text buffer in the viewer
 */
void TextViewer::LoadBuffer(const char *buffer)
{
   fView->LoadBuffer(buffer);
}

/* 
 * Load text from a file in the viewer. 
 */
void TextViewer::LoadFile(const char *file)
{
   fView->LoadFile(file);
}

/*
 * Load a TGText* buffer in the viewer.
 */
void TextViewer::SetText(const TGText *buffer)
{
   fView->SetText(new TGText((TGText *) buffer));
}

/* 
 * Called when closed via window manager action.
 */
void TextViewer::CloseWindow()
{
   delete this;
}

/*
 * Process messages
 */
Bool_t TextViewer::ProcessMessage(Long_t msg, Long_t, Long_t)
{
    // Process OK button.
  switch (GET_MSG(msg)) {
    case kC_COMMAND:
      switch (GET_SUBMSG(msg)) {
        case kCM_BUTTON:
            // Only one button and one action...
          delete this;
          break;
        default:
          break;
      }
      break;
    case kC_TEXTVIEW:
      switch (GET_SUBMSG(msg)) {
        case kTXT_CLOSE:
            // close window
          delete this;
          break;
        case kTXT_OPEN:
          SetTitle();
          break;
        case kTXT_SAVE:
          SetTitle();
          break;
        default:
          break;
      }
    default:
      break;
  }

  return kTRUE;
}
