#include <iostream.h>
#include <fstream>

void analyse_badchannel_new_tt6(int nchip, int cut_type, 
	      float ped_l, float rnoi_l, float cmnoi_l, float mxch_l,
	      float ped_h, float rnoi_h, float cmnoi_h, float mxch_h,
              const char* canvas_name) {
  char  hName[30], hTitle[30];
  char  hNameMaxChD[30], hNameMaxChS[30], hTitleD[30], hTitleS[30];
  float pedLow, pedHigh, noiseLow, noiseHigh;
  float calibLow, calibHigh;
  int   nChan = 128; 
  int   nStrip = nChan * nchip;

  // Define the Histogram for Bad Strip Flag 
  sprintf(hName, "bad_strip_flag_vs_str"); 
  sprintf(hTitle, "Bad Strip Flag vs Strip No.");
  TH1F* fThFlag =  new TH1F(hName, hTitle, nStrip, 0.5, nStrip+0.5);   

  for (int ich = 0; ich < nchip; ich++) {
    //
    // -- Define Maximum Charge Histogram
    //
    sprintf(hTitleD, "Maximum Charge for APV #%i", ich+1);
    sprintf(hTitleS, "Maximum Charge Vs Strip for APV #%i", ich+1);
    sprintf(hNameMaxChD, "Max_Charge_APV_%i", ich+1); 
    sprintf(hNameMaxChS, "Max_Charge_vs_strip_APV_%i", ich+1); 
  
    TH1F* fThMaxChD =  new TH1F(hNameMaxChD, hTitleD, 100, -150.0, 0.0);
    TH1F* fThMaxChS =  new TH1F(hNameMaxChS, hTitleS, nChan, 0.5, nChan+0.5);
    for (int k = 0; k < nChan; k++) {
      if (k<9) {
        sprintf(hName, "APV_%i_Strip_0%i", ich+1, k+1);
      } else {
        sprintf(hName, "APV_%i_Strip_%i", ich+1, k+1); 
      }   
      TH1 *fThCalib = (TH1 *) gROOT->FindObject(hName);
      if (fThCalib) {
        fThMaxChD->Fill(fThCalib->GetMinimum());  
        fThMaxChS->Fill(float(k+1), fThCalib->GetMinimum());
      }
    }  
    float calibMean = fThMaxChD->GetMean();
    float calibRms  = fThMaxChD->GetRMS();  
    //
    // -- Get Pedestal Histograms 
    //   
    sprintf(hName, "APV_%i_pedestal", ich+1); 
    TH1* fThPed1 = (TH1 *) gROOT->FindObject(hName);
    float pedMean = fThPed1->GetMean();
    float pedRms  = fThPed1->GetRMS();
    sprintf(hName, "APV_%i_ped_vs_str", ich+1); 
    TH1* fThPed2 = (TH1 *) gROOT->FindObject(hName);  
    //
    // -- Get Common Subtracted Noise Histograms 
    //   
    sprintf(hName, "APV_%i_cmnoise",ich+1); 
    TH1* fThCMNoise1 = (TH1 *) gROOT->FindObject(hName);
    float cmNoiseMean = fThCMNoise1->GetMean();
    float cmNoiseRms  = fThCMNoise1->GetRMS();
    sprintf(hName, "APV_%i_cmnoise_vs_str", ich+1); 
    TH1* fThCMNoise2 = (TH1 *) gROOT->FindObject(hName);  
    // 
    // -- Define Cuts 
    //
     pedLow      = ped_l;
     pedHigh     = ped_h;         
     NoiseLow    = noi_l;
     NoiseHigh   = noi_h;
      cmNoiseLow  = cmnoi_l;
      cmNoiseHigh = cmnoi_h;
      calibLow    = mxch_l;
      calibHigh   = mxch_h;
    }
    cout << " APV Number " << ich+1 << "(Mean Values and Lower, Higher Cuts)"<< endl;
    cout << " =========================================================== " << endl;
    cout << " Pedestal  ==========> " << pedMean << " " << pedLow  << " " << pedHigh << endl;
    cout << " Raw Noise ==========> " << rNoiseMean << " " << rNoiseLow  << " " << rNoiseHigh << endl;
    cout << " Common Mode Noise ==> " << cmNoiseMean << " " << cmNoiseLow  << " " << cmNoiseHigh << endl;
    cout << " Max Calib Charge  ==> " << calibMean << " " << calibLow  << " " << calibHigh << endl;
    //
    // -- Now Search for Bad Strips
    //
    for (int k = 0; k < nChan; k++) {  
      int fVal = 0;
      float pedVal      = fThPed2->GetBinContent(k+1);
      float rNoiseVal   = fThRNoise2->GetBinContent(k+1);
      float cmNoiseVal  = fThCMNoise2->GetBinContent(k+1);
      float maxCalVal = fThMaxChS->GetBinContent(k+1);
      if (pedVal > pedHigh         || pedVal < pedLow)         fVal |= (1<<0);
      if (rNoiseVal > rNoiseHigh   || rNoiseVal < rNoiseLow)   fVal |= (1<<1);
      if (cmNoiseVal > cmNoiseHigh || cmNoiseVal < cmNoiseLow) fVal |= (1<<2);
      if (maxCalVal  < calibHigh    || maxCalVal > calibLow)   fVal |= (1<<3);
      fThFlag->Fill(float((ich*128.0)+k+1), fVal);
    }
    delete fThMaxChD;
    delete fThMaxChS;
  }
  TCanvas *tc = (TCanvas *) gROOT->FindObject(canvas_name);
  if (tc) {
    gStyle->SetOptStat(0); 
    tc->Clear();   
    tc->Divide(1,1);
    tc->cd(1);
    fThFlag->Draw();
    tc->Update();
    tc->Modified(kTRUE);
  }
}

int calculate_mean_rms(TH1* fTH,int cutL,int cutH,float &fMean,float &fRMS){
  #include <vector>
  
  vector<float> fVal;
  Int_t nBin = fTh1->GetNbinsX();
  
  for (int i = 1; i < nBin+1; i++) {
    fVal.push_back(fTh1->GetBinContent(i));
  }
  sort(fVal.begin(), fVal.end());
  float sumVal   = 0.0;
  float sqSumVal = 0.0;
  sumVal   = accumulate((fVal.begin()+cutL), (fVal.end()-cutH), sumVal);
  sqSumVal = inner_product((fVal.begin()+cutL), (fVal.end()-cutH), 
                                    (fVal.begin()+cutL), sqSumVal);
  int nVal = fVal.size() - cutL - cutH;
  float avVal    = (nVal) ? sumVal/nVal   : 0.0;
  float sqAvVal  = (nVal) ? sqSumVal/nVal : 0.0;
  float rmsVal   = (sqAvVal - avVal*avVal > 0.0) 
             ? sqrt(sqAvVal - avVal*avVal) : 0.0;	  
}
