#include <iostream.h>
void analyse_channel_tt6(){
  int cut_type = 1;
  float cut_low[4];
  float cut_high[4];
  int nchip = 1;
  for (int i = 0; i < 4; i++) {
    cut_low[i] = 0.3;
    cut_high[i] = 0.2;
  }
  test(nchip, cut_type, cut_low, cut_high);
}
void test(int nchip, int cut_type, 
              const float* cut_low, const float* cut_high){
  char  hName[30], hTitle[30];
  char  hNameMaxChD[30], hNameMaxChS[30], hTitleD[30], hTitleS[30];
  float pedLow, pedHigh, rNoiseLow, rNoiseHigh, cmNoiseLow, cmNoiseHigh;
  float calibLow, calibHigh;
  int   nChan = 128; 
  //
  // -- Define Maximum Charge Histogram
  //
  sprintf(hTitleD, "Maximum Charge for APV #%i",nchip);
  sprintf(hTitleS, "Maximum Charge Vs Strip for APV #%i",nchip);
  sprintf(hNameMaxChD, "Max_Charge_APV_%i",nchip); 
  sprintf(hNameMaxChS, "Max_Charge_vs_strip_APV_%i",nchip); 
  
  TH1F* fThMaxChD =  new TH1F(hNameMaxChD, hTitleD, 100, -150.0, 0.0);
  TH1F* fThMaxChS =  new TH1F(hNameMaxChS, hTitleS, nChan, 0.5, nChan+0.5);
  for (int k = 0; k < nChan; k++) {
    if (k<9) {
      sprintf(hName, "APV_%i_Strip_0%i",nchip,k+1);
    } else {
      sprintf(hName, "APV_%i_Strip_%i",nchip,k+1); 
    }   
    TH1 *fThCalib = (TH1 *) gROOT->FindObject(hName);
    fThMaxChD->Fill(fThCalib->GetMinimum());  
    fThMaxChS->Fill(float(k+1), fThCalib->GetMinimum());
  }  
  float calibMean = fThMaxChD->GetMean();
  float calibRms  = fThMaxChD->GetRMS();  
  //
  // -- Get Pedestal Histograms 
  //   
  sprintf(hName, "APV_%i_pedestal",nchip); 
  TH1* fThPed1 = (TH1 *) gROOT->FindObject(hName);
  float pedMean = fThPed1->GetMean();
  float pedRms  = fThPed1->GetRMS();
  sprintf(hName, "APV_%i_ped_vs_str",nchip); 
  TH1* fThPed2 = (TH1 *) gROOT->FindObject(hName);  
  //
  // -- Get Raw Noise  Histograms 
  //   
  sprintf(hName, "APV_%i_rnoise",nchip); 
  TH1* fThRNoise1 = (TH1 *) gROOT->FindObject(hName);
  float rNoiseMean = fThRNoise1->GetMean();
  float rNoiseRms  = fThRNoise1->GetRMS();
  sprintf(hName, "APV_%i_rnoise_vs_str",nchip); 
  TH1* fThRNoise2 = (TH1 *) gROOT->FindObject(hName);  
  //
  // -- Get Common Subtracted Noise Histograms 
  //   
  sprintf(hName, "APV_%i_cmnoise",nchip); 
  TH1* fThCMNoise1 = (TH1 *) gROOT->FindObject(hName);
  float cmNoiseMean = fThCMNoise1->GetMean();
  float cmNoiseRms  = fThCMNoise1->GetRMS();
  sprintf(hName, "APV_%i_cmnoise_vs_str",nchip); 
  TH1* fThCMNoise2 = (TH1 *) gROOT->FindObject(hName);  
  // 
  // -- Define Cuts 
  //
  if (cut_type == 1) {
    pedLow      = pedMean - (pedMean * cut_low[0]);
    pedHigh     = pedMean + (pedMean * cut_high[0]);
    rNoiseLow   = rNoiseMean - (rNoiseMean * cut_low[1]);
    rNoiseHigh  = rNoiseMean + (rNoiseMean * cut_high[1]);
    cmNoiseLow  = cmNoiseMean - (cmNoiseMean * cut_low[2]);
    cmNoiseHigh = cmNoiseMean + (cmNoiseMean * cut_high[2]);
    calibLow    = calibMean - (calibMean * cut_low[3]);
    calibHigh   = calibMean + (calibMean * cut_high[3]);
  } else if (cut_type == 2) {
    pedLow      = pedMean - (pedRms * cut_low[0]);
    pedHigh     = pedMean + (pedRms * cut_high[0]);     
    rNoiseLow   = rNoiseMean - (rNoiseRms * cut_low[1]);
    rNoiseHigh  = rNoiseMean + (rNoiseRms * cut_high[1]);
    cmNoiseLow  = cmNoiseMean - (cmNoiseRms * cut_low[2]);
    cmNoiseHigh = cmNoiseMean + (cmNoiseRms * cut_high[2]);
    calibLow    = calibMean - (calibRms * cut_low[3]);
    calibHigh   = calibMean + (calibRms * cut_high[3]);
  } else if (cut_type == 3) {
    pedLow      = cut_low[0];
    pedHigh     = cut_high[0];         
    rNoiseLow   = cut_low[1];
    rNoiseHigh  = cut_high[1];
    cmNoiseLow  = cut_low[2];
    cmNoiseHigh = cut_high[2];
    calibLow    = cut_low[3];
    calibHigh   = cut_high[3];
  }
  cout << "Pedestal Mean and Cuts "  << pedMean << " " << pedLow  << " " << pedHigh << endl;
  cout << "Raw Noise Mean and Cuts "   << rNoiseMean << " " << rNoiseLow  << " " << rNoiseHigh << endl;
  cout << "CM Noise Mean and Cuts "    << cmNoiseMean << " " << cmNoiseLow  << " " << cmNoiseHigh << endl;
  cout << "Calibration Mean and Cuts " << calibMean << " " << calibLow  << " " << calibHigh << endl;
  //
  // -- Search for Bad Strip
  //
  sprintf(hName, "APV_%i_StripFlag_vs_str",nchip); 
  sprintf(hTitle, "APV #%i StripFlag",nchip);
  TH1F* fThFlag =  new TH1F(hName, hTitle, nChan, 0.5, nChan+0.5);   
  for (int k = 0; k < nChan; k++) {  
    int fVal = 0;
    float pedVal      = fThPed2->GetBinContent(k+1);
    float rNoiseVal   = fThRNoise2->GetBinContent(k+1);
    float cmNoiseVal  = fThCMNoise2->GetBinContent(k+1);
    float maxCalVal = fThMaxChS->GetBinContent(k+1);
    if (pedVal > pedHigh         || pedVal < pedLow)         fVal |= (1<<0);
    if (rNoiseVal > rNoiseHigh   || rNoiseVal < rNoiseLow)   {
      fVal |= (1<<1);
      cout << "Raw Noise " << rNoiseVal << " " << k+1 << endl;
    }
    if (cmNoiseVal > cmNoiseHigh || cmNoiseVal < cmNoiseLow) fVal |= (1<<2);
    if (maxCalVal  < calibHigh    || maxCalVal > calibLow)    fVal |= (1<<3);
    fThFlag->Fill(float(k+1), fVal);
  }
  TCanvas *tc = (TCanvas *) gROOT->FindObject("Histogram Canvas");
  if (tc) {
    tc->Divide(1,1);
    tc->cd(1);
    fThFlag->Draw();
    tc->Update();
    tc->Modified(kTRUE);
  }
  delete fThMaxChD;
  delete fThMaxChS;
}
